@@ -4,7 +4,6 @@ require 'rack/builder'
 require 'rack/accept'
 require 'rack/auth/basic'
 require 'rack/auth/digest/md5'
-require 'hashie'
 require 'set'
 require 'active_support/version'
 require 'active_support/core_ext/hash/indifferent_access'
@@ -14,11 +13,10 @@ require 'active_support/core_ext/array/wrap'
 require 'active_support/core_ext/hash/deep_merge'
 require 'active_support/core_ext/hash/reverse_merge'
 require 'active_support/core_ext/hash/except'
+require 'active_support/core_ext/hash/slice'
 require 'active_support/core_ext/hash/conversions'
 require 'active_support/dependencies/autoload'
 require 'active_support/notifications'
-require 'multi_json'
-require 'multi_xml'
 require 'i18n'
 require 'thread'
 
@@ -27,7 +25,7 @@ require 'virtus'
 I18n.load_path << File.expand_path('../grape/locale/en.yml', __FILE__)
 
 module Grape
-  extend ActiveSupport::Autoload
+  extend ::ActiveSupport::Autoload
 
   eager_autoload do
     autoload :API
@@ -45,19 +43,22 @@ module Grape
     autoload :Parser
     autoload :Request
     autoload :Env, 'grape/util/env'
+    autoload :Json, 'grape/util/json'
+    autoload :Xml, 'grape/util/xml'
   end
 
   module Http
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     eager_autoload do
       autoload :Headers
     end
   end
 
   module Exceptions
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Base
     autoload :Validation
+    autoload :ValidationArrayErrors
     autoload :ValidationErrors
     autoload :MissingVendorOption
     autoload :MissingMimeType
@@ -77,16 +78,38 @@ module Grape
     autoload :MethodNotAllowed
   end
 
+  module Extensions
+    extend ::ActiveSupport::Autoload
+
+    autoload :DeepMergeableHash
+    autoload :DeepSymbolizeHash
+    autoload :DeepHashWithIndifferentAccess
+    autoload :Hash
+
+    module ActiveSupport
+      extend ::ActiveSupport::Autoload
+
+      autoload :HashWithIndifferentAccess
+    end
+
+    module Hashie
+      extend ::ActiveSupport::Autoload
+
+      autoload :Mash
+    end
+  end
+
   module Middleware
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Base
     autoload :Versioner
     autoload :Formatter
     autoload :Error
     autoload :Globals
+    autoload :Stack
 
     module Auth
-      extend ActiveSupport::Autoload
+      extend ::ActiveSupport::Autoload
       autoload :Base
       autoload :DSL
       autoload :StrategyInfo
@@ -94,7 +117,7 @@ module Grape
     end
 
     module Versioner
-      extend ActiveSupport::Autoload
+      extend ::ActiveSupport::Autoload
       autoload :Path
       autoload :Header
       autoload :Param
@@ -103,16 +126,17 @@ module Grape
   end
 
   module Util
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :InheritableValues
     autoload :StackableValues
+    autoload :ReverseStackableValues
     autoload :InheritableSetting
     autoload :StrictHashConfiguration
     autoload :Registrable
   end
 
   module ErrorFormatter
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Base
     autoload :Json
     autoload :Txt
@@ -120,7 +144,7 @@ module Grape
   end
 
   module Formatter
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Json
     autoload :SerializableHash
     autoload :Txt
@@ -128,13 +152,13 @@ module Grape
   end
 
   module Parser
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Json
     autoload :Xml
   end
 
   module DSL
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     eager_autoload do
       autoload :API
       autoload :Callbacks
@@ -153,17 +177,17 @@ module Grape
   end
 
   class API
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Helpers
   end
 
   module Presenters
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :Presenter
   end
 
   module ServeFile
-    extend ActiveSupport::Autoload
+    extend ::ActiveSupport::Autoload
     autoload :FileResponse
     autoload :FileBody
     autoload :SendfileResponse
@@ -175,6 +199,7 @@ require 'grape/util/content_types'
 require 'grape/validations/validators/base'
 require 'grape/validations/attributes_iterator'
 require 'grape/validations/validators/allow_blank'
+require 'grape/validations/validators/as'
 require 'grape/validations/validators/at_least_one_of'
 require 'grape/validations/validators/coerce'
 require 'grape/validations/validators/default'
@@ -183,8 +208,10 @@ require 'grape/validations/validators/mutual_exclusion'
 require 'grape/validations/validators/presence'
 require 'grape/validations/validators/regexp'
 require 'grape/validations/validators/values'
+require 'grape/validations/validators/except_values'
 require 'grape/validations/params_scope'
 require 'grape/validations/validators/all_or_none'
 require 'grape/validations/types'
+require 'grape/validations/validator_factory'
 
 require 'grape/version'
