@@ -8,6 +8,32 @@ module Grape
     module Parameters
       extend ActiveSupport::Concern
 
+      # Set the module used to build the request.params.
+      #
+      # @param build_with the ParamBuilder module to use when building request.params
+      #   Available builders are:
+      #
+      #     * Grape::Extensions::ActiveSupport::HashWithIndifferentAccess::ParamBuilder (default)
+      #     * Grape::Extensions::Hash::ParamBuilder
+      #     * Grape::Extensions::Hashie::Mash::ParamBuilder
+      #
+      # @example
+      #
+      #     require 'grape/extenstions/hashie_mash'
+      #     class API < Grape::API
+      #       desc "Get collection"
+      #       params do
+      #         build_with Grape::Extensions::Hashie::Mash::ParamBuilder
+      #         requires :user_id, type: Integer
+      #       end
+      #       get do
+      #         params['user_id']
+      #       end
+      #     end
+      def build_with(build_with = nil)
+        @api.namespace_inheritable(:build_params_with, build_with)
+      end
+
       # Include reusable params rules among current.
       # You can define reusable params with helpers method.
       #
@@ -34,13 +60,13 @@ module Grape
         options = names.extract_options!
         names.each do |name|
           params_block = named_params.fetch(name) do
-            fail "Params :#{name} not found!"
+            raise "Params :#{name} not found!"
           end
           instance_exec(options, &params_block)
         end
       end
-      alias_method :use_scope, :use
-      alias_method :includes, :use
+      alias use_scope use
+      alias includes use
 
       # Require one or more parameters for the current endpoint.
       #
@@ -99,13 +125,13 @@ module Grape
 
         opts = attrs.extract_options!.clone
         opts[:presence] = { value: true, message: opts[:message] }
+        opts = @group.merge(opts) if @group
 
         if opts[:using]
           require_required_and_optional_fields(attrs.first, opts)
         else
           validate_attributes(attrs, opts, &block)
-
-          block_given? ? new_scope(orig_attrs, &block) : push_declared_params(attrs)
+          block_given? ? new_scope(orig_attrs, &block) : push_declared_params(attrs, opts.slice(:as))
         end
       end
 
@@ -118,11 +144,12 @@ module Grape
 
         opts = attrs.extract_options!.clone
         type = opts[:type]
+        opts = @group.merge(opts) if @group
 
         # check type for optional parameter group
         if attrs && block_given?
-          fail Grape::Exceptions::MissingGroupTypeError.new if type.nil?
-          fail Grape::Exceptions::UnsupportedGroupTypeError.new unless Grape::Validations::Types.group?(type)
+          raise Grape::Exceptions::MissingGroupTypeError.new if type.nil?
+          raise Grape::Exceptions::UnsupportedGroupTypeError.new unless Grape::Validations::Types.group?(type)
         end
 
         if opts[:using]
@@ -130,10 +157,17 @@ module Grape
         else
           validate_attributes(attrs, opts, &block)
 
-          block_given? ? new_scope(orig_attrs, true, &block) : push_declared_params(attrs)
+          block_given? ? new_scope(orig_attrs, true, &block) : push_declared_params(attrs, opts.slice(:as))
         end
       end
 
+      # Define common settings for one or more parameters
+      # @param (see #requires)
+      # @option (see #requires)
+      def with(*attrs, &block)
+        new_group_scope(attrs.clone, &block)
+      end
+
       # Disallow the given parameters to be present in the same request.
       # @param attrs [*Symbol] parameters to validate
       def mutually_exclusive(*attrs)
@@ -167,7 +201,8 @@ module Grape
       # @yield a parameter definition DSL
       def given(*attrs, &block)
         attrs.each do |attr|
-          fail Grape::Exceptions::UnknownParameter.new(attr) unless declared_param?(attr)
+          proxy_attr = attr.is_a?(Hash) ? attr.keys[0] : attr
+          raise Grape::Exceptions::UnknownParameter.new(proxy_attr) unless declared_param?(proxy_attr)
         end
         new_lateral_scope(dependent_on: attrs, &block)
       end
@@ -181,22 +216,26 @@ module Grape
         @declared_params.flatten.include?(param)
       end
 
-      alias_method :group, :requires
+      alias group requires
+
+      def map_params(params, element)
+        if params.is_a?(Array)
+          params.map do |el|
+            map_params(el, element)
+          end
+        elsif params.is_a?(Hash)
+          params[element] || {}
+        else
+          {}
+        end
+      end
 
       # @param params [Hash] initial hash of parameters
       # @return hash of parameters relevant for the current scope
       # @api private
       def params(params)
         params = @parent.params(params) if @parent
-        if @element
-          if params.is_a?(Array)
-            params = params.flat_map { |el| el[@element] || {} }
-          elsif params.is_a?(Hash)
-            params = params[@element] || {}
-          else
-            params = {}
-          end
-        end
+        params = map_params(params, @element) if @element
         params
       end
     end
