@@ -7,7 +7,7 @@ module Grape
       include Grape::DSL::Configuration
 
       module ClassMethods
-        attr_reader :endpoints, :routes
+        attr_reader :endpoints
 
         # Specify an API version.
         #
@@ -30,20 +30,21 @@ module Grape
           if args.any?
             options = args.extract_options!
             options = options.reverse_merge(using: :path)
+            requested_versions = args.flatten
 
-            fail Grape::Exceptions::MissingVendorOption.new if options[:using] == :header && !options.key?(:vendor)
+            raise Grape::Exceptions::MissingVendorOption.new if options[:using] == :header && !options.key?(:vendor)
 
-            @versions = versions | args
+            @versions = versions | requested_versions
 
             if block_given?
               within_namespace do
-                namespace_inheritable(:version, args)
+                namespace_inheritable(:version, requested_versions)
                 namespace_inheritable(:version_options, options)
 
                 instance_eval(&block)
               end
             else
-              namespace_inheritable(:version, args)
+              namespace_inheritable(:version, requested_versions)
               namespace_inheritable(:version_options, options)
             end
           end
@@ -98,6 +99,7 @@ module Grape
               method: :any,
               path: path,
               app: app,
+              route_options: { anchor: false },
               forward_match: !app.respond_to?(:inheritable_setting),
               for: self
             )
@@ -117,13 +119,14 @@ module Grape
         #     end
         #   end
         def route(methods, paths = ['/'], route_options = {}, &block)
+          methods = '*' if methods == :any
           endpoint_options = {
             method: methods,
             path: paths,
             for: self,
-            route_options: ({
+            route_options: {
               params: namespace_stackable_with_hash(:params) || {}
-            }).deep_merge(route_setting(:description) || {}).deep_merge(route_options || {})
+            }.deep_merge(route_setting(:description) || {}).deep_merge(route_options || {})
           }
 
           new_endpoint = Grape::Endpoint.new(inheritable_setting, endpoint_options, &block)
@@ -133,7 +136,7 @@ module Grape
           reset_validations!
         end
 
-        %w(get post put head delete options patch).each do |meth|
+        %w[get post put head delete options patch].each do |meth|
           define_method meth do |*args, &block|
             options = args.extract_options!
             paths = args.first || ['/']
@@ -170,10 +173,10 @@ module Grape
           end
         end
 
-        alias_method :group, :namespace
-        alias_method :resource, :namespace
-        alias_method :resources, :namespace
-        alias_method :segment, :namespace
+        alias group namespace
+        alias resource namespace
+        alias resources namespace
+        alias segment namespace
 
         # An array of API routes.
         def routes
@@ -197,6 +200,7 @@ module Grape
         # @option options [Regexp] You may supply a regular expression that the declared parameter must meet.
         def route_param(param, options = {}, &block)
           options = options.dup
+
           options[:requirements] = {
             param.to_sym => options[:requirements]
           } if options[:requirements].is_a?(Regexp)
@@ -204,6 +208,7 @@ module Grape
           Grape::Validations::ParamsScope.new(api: self) do
             requires param, type: options[:type]
           end if options.key?(:type)
+
           namespace(":#{param}", options, &block)
         end
 
