@@ -2,18 +2,18 @@ require 'active_support/concern'
 
 module Grape
   module DSL
-    # Keeps track of settings (impemented as key-value pairs, grouped by
+    # Keeps track of settings (implemented as key-value pairs, grouped by
     # types), in two contexts: top-level settings which apply globally no
     # matter where they're defined, and inheritable settings which apply only
     # in the current scope and scopes nested under it.
     module Settings
       extend ActiveSupport::Concern
 
-      attr_accessor :inheritable_setting, :top_level_setting
+      attr_writer :inheritable_setting, :top_level_setting
 
       # Fetch our top-level settings, which apply to all endpoints in the API.
       def top_level_setting
-        @top_level_setting ||= Grape::Util::InheritableSetting.new
+        @top_level_setting ||= build_top_level_setting
       end
 
       # Fetch our current inheritable settings, which are inherited by
@@ -71,7 +71,7 @@ module Grape
 
       # (see #unset_global_setting)
       def unset_namespace_setting(key)
-        unset :namespace_setting, key
+        unset :namespace, key
       end
 
       # (see #global_setting)
@@ -94,6 +94,28 @@ module Grape
         get_or_set :namespace_stackable, key, value
       end
 
+      def namespace_reverse_stackable(key, value = nil)
+        get_or_set :namespace_reverse_stackable, key, value
+      end
+
+      def namespace_stackable_with_hash(key)
+        settings = get_or_set :namespace_stackable, key, nil
+        return if settings.blank?
+        settings.each_with_object({}) { |value, result| result.deep_merge!(value) }
+      end
+
+      def namespace_reverse_stackable_with_hash(key)
+        settings = get_or_set :namespace_reverse_stackable, key, nil
+        return if settings.blank?
+        result = {}
+        settings.each do |setting|
+          setting.each do |field, value|
+            result[field] ||= value
+          end
+        end
+        result
+      end
+
       # (see #unset_global_setting)
       def unset_namespace_stackable(key)
         unset :namespace_stackable, key
@@ -106,7 +128,7 @@ module Grape
 
       # (see #unset_global_setting)
       def unset_api_class_setting(key)
-        unset :api_class_setting, key
+        unset :api_class, key
       end
 
       # Fork our inheritable settings to a new instance, copied from our
@@ -140,6 +162,18 @@ module Grape
 
         result
       end
+
+      private
+
+      # Builds the current class :inheritable_setting. If available, it inherits from
+      # the superclass's :inheritable_setting.
+      def build_top_level_setting
+        Grape::Util::InheritableSetting.new.tap do |setting|
+          if defined?(superclass) && superclass.respond_to?(:inheritable_setting) && superclass != Grape::API
+            setting.inherit_from superclass.inheritable_setting
+          end
+        end
+      end
     end
   end
 end
