@@ -17,8 +17,8 @@ module Grape
         builtin_formatters.merge(default_elements).merge(options[:error_formatters] || {})
       end
 
-      def formatter_for(api_format, options = {})
-        spec = formatters(options)[api_format]
+      def formatter_for(api_format, **options)
+        spec = formatters(**options)[api_format]
         case spec
         when nil
           options[:default_error_formatter] || Grape::ErrorFormatter::Txt
