@@ -7,13 +7,14 @@ module Grape
 
       attr_reader :errors
 
-      def initialize(args = {})
+      def initialize(errors: [], headers: {}, **_options)
         @errors = {}
-        args[:errors].each do |validation_error|
+        errors.each do |validation_error|
           @errors[validation_error.params] ||= []
           @errors[validation_error.params] << validation_error
         end
-        super message: full_messages.join(', '), status: 400
+
+        super message: full_messages.join(', '), status: 400, headers: headers
       end
 
       def each
@@ -24,7 +25,7 @@ module Grape
         end
       end
 
-      def as_json(_opts = {})
+      def as_json(**_opts)
         errors.map do |k, v|
           {
             params: k,
@@ -33,16 +34,16 @@ module Grape
         end
       end
 
-      def to_json(_opts = {})
+      def to_json(**_opts)
         as_json.to_json
       end
 
-      private
-
       def full_messages
         map { |attributes, error| full_message(attributes, error) }.uniq
       end
 
+      private
+
       def full_message(attributes, error)
         I18n.t(
           'grape.errors.format'.to_sym,
