@@ -17,8 +17,8 @@ module Grape
         builtin_formmaters.merge(default_elements).merge(options[:formatters] || {})
       end
 
-      def formatter_for(api_format, options = {})
-        spec = formatters(options)[api_format]
+      def formatter_for(api_format, **options)
+        spec = formatters(**options)[api_format]
         case spec
         when nil
           ->(obj, _env) { obj }
