@@ -4,27 +4,28 @@ module Grape
       class << self
         def call(object, _env)
           return object if object.is_a?(String)
-          return MultiJson.dump(serialize(object)) if serializable?(object)
+          return ::Grape::Json.dump(serialize(object)) if serializable?(object)
           return object.to_json if object.respond_to?(:to_json)
-          MultiJson.dump(object)
+          ::Grape::Json.dump(object)
         end
 
         private
 
         def serializable?(object)
-          object.respond_to?(:serializable_hash) || object.is_a?(Array) && !object.map { |o| o.respond_to? :serializable_hash }.include?(false) || object.is_a?(Hash)
+          object.respond_to?(:serializable_hash) || object.is_a?(Array) && object.all? { |o| o.respond_to? :serializable_hash } || object.is_a?(Hash)
         end
 
         def serialize(object)
           if object.respond_to? :serializable_hash
             object.serializable_hash
-          elsif object.is_a?(Array) && !object.map { |o| o.respond_to? :serializable_hash }.include?(false)
+          elsif object.is_a?(Array) && object.all? { |o| o.respond_to? :serializable_hash }
             object.map(&:serializable_hash)
           elsif object.is_a?(Hash)
-            object.inject({}) do |h, (k, v)|
+            h = {}
+            object.each_pair do |k, v|
               h[k] = serialize(v)
-              h
             end
+            h
           else
             object
           end
