@@ -10,9 +10,10 @@ module Grape
 
       # @param [Rack Application] app The standard argument for a Rack middleware.
       # @param [Hash] options A hash of options, simply stored for use by subclasses.
-      def initialize(app, options = {})
+      def initialize(app, **options)
         @app = app
-        @options = default_options.merge(options)
+        @options = default_options.merge(**options)
+        @app_response = nil
       end
 
       def default_options
@@ -44,14 +45,12 @@ module Grape
 
       # @abstract
       # Called before the application is called in the middleware lifecycle.
-      def before
-      end
+      def before; end
 
       # @abstract
       # Called after the application is called in the middleware lifecycle.
       # @return [Response, nil] a Rack SPEC response or nil to call the application afterwards.
-      def after
-      end
+      def after; end
 
       def response
         return @app_response if @app_response.is_a?(Rack::Response)
