@@ -8,29 +8,37 @@ module Grape
           default_status: 500, # default status returned on error
           default_message: '',
           format: :txt,
+          helpers: nil,
           formatters: {},
           error_formatters: {},
           rescue_all: false, # true to rescue all exceptions
+          rescue_grape_exceptions: false,
           rescue_subclasses: true, # rescue subclasses of exceptions listed
-          rescue_options: { backtrace: false }, # true to display backtrace
+          rescue_options: {
+            backtrace: false, # true to display backtrace, true to let Grape handle Grape::Exceptions
+            original_exception: false, # true to display exception
+          },
           rescue_handlers: {}, # rescue handler blocks
           base_only_rescue_handlers: {}, # rescue handler blocks rescuing only the base class
           all_rescue_handler: nil # rescue handler block to rescue from all exceptions
         }
       end
 
+      def initialize(app, **options)
+        super
+        self.class.send(:include, @options[:helpers]) if @options[:helpers]
+      end
+
       def call!(env)
         @env = env
 
-        inject_helpers!
-
         begin
           error_response(catch(:error) do
             return @app.call(@env)
           end)
         rescue StandardError => e
           is_rescuable = rescuable?(e.class)
-          if e.is_a?(Grape::Exceptions::Base) && !is_rescuable
+          if e.is_a?(Grape::Exceptions::Base) && (!is_rescuable || rescuable_by_grape?(e.class))
             handler = ->(arg) { error_response(arg) }
           else
             raise unless is_rescuable
@@ -45,50 +53,40 @@ module Grape
         handler = options[:rescue_handlers].find(-> { [] }) { |error, _| klass <= error }[1]
         handler ||= options[:base_only_rescue_handlers][klass]
         handler ||= options[:all_rescue_handler]
-        with_option = options[:rescue_options][:with]
-        if with_option.instance_of?(Symbol)
-          if respond_to?(with_option)
-            handler ||= self.class.instance_method(with_option).bind(self)
-          else
-            fail NoMethodError, "undefined method `#{with_option}'"
-          end
+
+        if handler.instance_of?(Symbol)
+          raise NoMethodError, "undefined method `#{handler}'" unless respond_to?(handler)
+          handler = self.class.instance_method(handler).bind(self)
         end
+
         handler
       end
 
       def rescuable?(klass)
         return false if klass == Grape::Exceptions::InvalidVersionHeader
-        options[:rescue_all] || (options[:rescue_handlers] || []).any? { |error, _handler| klass <= error } || (options[:base_only_rescue_handlers] || []).include?(klass)
+        rescue_all? || rescue_class_or_its_ancestor?(klass) || rescue_with_base_only_handler?(klass)
+      end
+
+      def rescuable_by_grape?(klass)
+        return false if klass == Grape::Exceptions::InvalidVersionHeader
+        options[:rescue_grape_exceptions]
       end
 
       def exec_handler(e, &handler)
-        if handler.lambda? && handler.arity == 0
+        if handler.lambda? && handler.arity.zero?
           instance_exec(&handler)
         else
           instance_exec(e, &handler)
         end
       end
 
-      def inject_helpers!
-        return if helpers_available?
-        endpoint = @env['api.endpoint']
-        self.class.instance_eval do
-          include endpoint.send(:helpers)
-        end if endpoint.is_a?(Grape::Endpoint)
-        @helpers = true
-      end
-
-      def helpers_available?
-        @helpers
-      end
-
-      def error!(message, status = options[:default_status], headers = {}, backtrace = [])
+      def error!(message, status = options[:default_status], headers = {}, backtrace = [], original_exception = nil)
         headers = headers.reverse_merge(Grape::Http::Headers::CONTENT_TYPE => content_type)
-        rack_response(format_message(message, backtrace), status, headers)
+        rack_response(format_message(message, backtrace, original_exception), status, headers)
       end
 
       def handle_error(e)
-        error_response(message: e.message, backtrace: e.backtrace)
+        error_response(message: e.message, backtrace: e.backtrace, original_exception: e)
       end
 
       # TODO: This method is deprecated. Refactor out.
@@ -97,19 +95,38 @@ module Grape
         message = error[:message] || options[:default_message]
         headers = { Grape::Http::Headers::CONTENT_TYPE => content_type }
         headers.merge!(error[:headers]) if error[:headers].is_a?(Hash)
-        backtrace = error[:backtrace] || []
-        rack_response(format_message(message, backtrace), status, headers)
+        backtrace = error[:backtrace] || error[:original_exception] && error[:original_exception].backtrace || []
+        original_exception = error.is_a?(Exception) ? error : error[:original_exception] || nil
+        rack_response(format_message(message, backtrace, original_exception), status, headers)
       end
 
       def rack_response(message, status = options[:default_status], headers = { Grape::Http::Headers::CONTENT_TYPE => content_type })
         Rack::Response.new([message], status, headers).finish
       end
 
-      def format_message(message, backtrace)
+      def format_message(message, backtrace, original_exception = nil)
         format = env[Grape::Env::API_FORMAT] || options[:format]
         formatter = Grape::ErrorFormatter.formatter_for(format, options)
-        throw :error, status: 406, message: "The requested format '#{format}' is not supported." unless formatter
-        formatter.call(message, backtrace, options, env)
+        throw :error,
+              status: 406,
+              message: "The requested format '#{format}' is not supported.",
+              backtrace: backtrace,
+              original_exception: original_exception unless formatter
+        formatter.call(message, backtrace, options, env, original_exception)
+      end
+
+      private
+
+      def rescue_all?
+        options[:rescue_all]
+      end
+
+      def rescue_class_or_its_ancestor?(klass)
+        (options[:rescue_handlers] || []).any? { |error, _handler| klass <= error }
+      end
+
+      def rescue_with_base_only_handler?(klass)
+        (options[:base_only_rescue_handlers] || []).include?(klass)
       end
     end
   end
