@@ -45,7 +45,7 @@ module Grape
           Rack::Response.new(bodymap, status, headers)
         end
       rescue Grape::Exceptions::InvalidFormatter => e
-        throw :error, status: 500, message: e.message
+        throw :error, status: 500, message: e.message, backtrace: e.backtrace, original_exception: e
       end
 
       def fetch_formatter(headers, options)
@@ -74,7 +74,7 @@ module Grape
         return unless
           (request.post? || request.put? || request.patch? || request.delete?) &&
           (!request.form_data? || !request.media_type) &&
-          (!request.parseable_data?) &&
+          !request.parseable_data? &&
           (request.content_length.to_i > 0 || request.env[Grape::Http::Headers::HTTP_TRANSFER_ENCODING] == CHUNKED)
 
         return unless (input = env[Grape::Env::RACK_INPUT])
@@ -82,7 +82,7 @@ module Grape
         input.rewind
         body = env[Grape::Env::API_REQUEST_INPUT] = input.read
         begin
-          read_rack_input(body) if body && body.length > 0
+          read_rack_input(body) if body && !body.empty?
         ensure
           input.rewind
         end
@@ -90,31 +90,30 @@ module Grape
 
       # store parsed input in env['api.request.body']
       def read_rack_input(body)
-        fmt = mime_types[request.media_type] if request.media_type
-        fmt ||= options[:default_format]
-        if content_type_for(fmt)
-          parser = Grape::Parser.parser_for fmt, options
-          if parser
-            begin
-              body = (env[Grape::Env::API_REQUEST_BODY] = parser.call(body, env))
-              if body.is_a?(Hash)
-                if env[Grape::Env::RACK_REQUEST_FORM_HASH]
-                  env[Grape::Env::RACK_REQUEST_FORM_HASH] = env[Grape::Env::RACK_REQUEST_FORM_HASH].merge(body)
-                else
-                  env[Grape::Env::RACK_REQUEST_FORM_HASH] = body
-                end
-                env[Grape::Env::RACK_REQUEST_FORM_INPUT] = env[Grape::Env::RACK_INPUT]
-              end
-            rescue Grape::Exceptions::Base => e
-              raise e
-            rescue StandardError => e
-              throw :error, status: 400, message: e.message
+        fmt = request.media_type ? mime_types[request.media_type] : options[:default_format]
+
+        unless content_type_for(fmt)
+          throw :error, status: 406, message: "The requested content-type '#{request.media_type}' is not supported."
+        end
+        parser = Grape::Parser.parser_for fmt, options
+        if parser
+          begin
+            body = (env[Grape::Env::API_REQUEST_BODY] = parser.call(body, env))
+            if body.is_a?(Hash)
+              env[Grape::Env::RACK_REQUEST_FORM_HASH] = if env[Grape::Env::RACK_REQUEST_FORM_HASH]
+                                                          env[Grape::Env::RACK_REQUEST_FORM_HASH].merge(body)
+                                                        else
+                                                          body
+                                                        end
+              env[Grape::Env::RACK_REQUEST_FORM_INPUT] = env[Grape::Env::RACK_INPUT]
             end
-          else
-            env[Grape::Env::API_REQUEST_BODY] = body
+          rescue Grape::Exceptions::Base => e
+            raise e
+          rescue StandardError => e
+            throw :error, status: 400, message: e.message, backtrace: e.backtrace, original_exception: e
           end
         else
-          throw :error, status: 406, message: "The requested content-type '#{request.media_type}' is not supported."
+          env[Grape::Env::API_REQUEST_BODY] = body
         end
       end
 
@@ -168,8 +167,8 @@ module Grape
         vendor_prefix_pattern = /vnd\.[^+]+\+/
 
         accept.scan(accept_into_mime_and_quality)
-          .sort_by { |_, quality_preference| -quality_preference.to_f }
-          .flat_map { |mime, _| [mime, mime.sub(vendor_prefix_pattern, '')] }
+              .sort_by { |_, quality_preference| -quality_preference.to_f }
+              .flat_map { |mime, _| [mime, mime.sub(vendor_prefix_pattern, '')] }
       end
     end
   end
