@@ -3,8 +3,8 @@ module Grape
     module Json
       class << self
         def call(object, _env)
-          MultiJson.load(object)
-        rescue MultiJson::ParseError
+          ::Grape::Json.load(object)
+        rescue ::Grape::Json::ParseError
           # handle JSON parsing errors via the rescue handlers or provide error message
           raise Grape::Exceptions::InvalidMessageBody, 'application/json'
         end
