@@ -2,6 +2,7 @@ module Grape
   module Validations
     class ParamsScope
       attr_accessor :element, :parent, :index
+      attr_reader :type
 
       include Grape::DSL::Parameters
 
@@ -15,6 +16,7 @@ module Grape
       # @option opts :optional [Boolean] whether or not this scope needs to have
       #   any parameters set or not
       # @option opts :type [Class] a type meant to govern this scope (deprecated)
+      # @option opts :type [Hash] group options for this scope
       # @option opts :dependent_on [Symbol] if present, this scope should only
       #   validate if this param is present in the parent scope
       # @yield the instance context, open for parameter definitions
@@ -24,8 +26,10 @@ module Grape
         @api          = opts[:api]
         @optional     = opts[:optional] || false
         @type         = opts[:type]
+        @group        = opts[:group] || {}
         @dependent_on = opts[:dependent_on]
         @declared_params = []
+        @index = nil
 
         instance_eval(&block) if block_given?
 
@@ -35,32 +39,51 @@ module Grape
       # @return [Boolean] whether or not this entire scope needs to be
       #   validated
       def should_validate?(parameters)
-        return false if @optional && params(parameters).respond_to?(:all?) && params(parameters).all?(&:blank?)
-        @dependent_on.each do |dependency|
-          return false if params(parameters).try(:[], dependency).blank?
-        end if @dependent_on
+        return false if @optional && (params(parameters).blank? || all_element_blank?(parameters))
+
         return true if parent.nil?
         parent.should_validate?(parameters)
       end
 
+      def meets_dependency?(params, request_params)
+        if @parent.present? && !@parent.meets_dependency?(@parent.params(request_params), request_params)
+          return false
+        end
+
+        return true unless @dependent_on
+
+        params = params.with_indifferent_access
+
+        @dependent_on.each do |dependency|
+          if dependency.is_a?(Hash)
+            dependency_key = dependency.keys[0]
+            proc = dependency.values[0]
+            return false unless proc.call(params.try(:[], dependency_key))
+          elsif params.respond_to?(:key?) && params.try(:[], dependency).blank?
+            return false
+          end
+        end
+
+        true
+      end
+
       # @return [String] the proper attribute name, with nesting considered.
-      def full_name(name)
-        case
-        when nested?
+      def full_name(name, index: nil)
+        if nested?
           # Find our containing element's name, and append ours.
-          "#{@parent.full_name(@element)}#{parent_index}[#{name}]"
-        when lateral?
-          # Find the name of the element as if it was at the
-          # same nesting level as our parent.
-          @parent.full_name(name)
+          [@parent.full_name(@element), [@index || index, name].map(&method(:brackets))].compact.join
+        elsif lateral?
+          # Find the name of the element as if it was at the same nesting level
+          # as our parent. We need to forward our index upward to achieve this.
+          @parent.full_name(name, index: @index)
         else
           # We must be the root scope, so no prefix needed.
           name.to_s
         end
       end
 
-      def parent_index
-        "[#{@parent.index}]" if @parent.present? && @parent.index.present?
+      def brackets(val)
+        "[#{val}]" if val
       end
 
       # @return [Boolean] whether or not this scope is the root-level scope
@@ -91,10 +114,15 @@ module Grape
 
       # Adds a parameter declaration to our list of validations.
       # @param attrs [Array] (see Grape::DSL::Parameters#requires)
-      def push_declared_params(attrs)
+      def push_declared_params(attrs, **opts)
         if lateral?
           @parent.push_declared_params(attrs)
         else
+          if opts && opts[:as]
+            @api.route_setting(:aliased_params, @api.route_setting(:aliased_params) || [])
+            @api.route_setting(:aliased_params) << { attrs.first => opts[:as] }
+          end
+
           @declared_params.concat attrs
         end
       end
@@ -111,7 +139,7 @@ module Grape
         end
         required_fields.each do |field|
           field_opts = opts[:using][field]
-          fail ArgumentError, "required field not exist: #{field}" unless field_opts
+          raise ArgumentError, "required field not exist: #{field}" unless field_opts
           requires(field, field_opts)
         end
         optional_fields.each do |field|
@@ -146,8 +174,8 @@ module Grape
         # if required params are grouped and no type or unsupported type is provided, raise an error
         type = attrs[1] ? attrs[1][:type] : nil
         if attrs.first && !optional
-          fail Grape::Exceptions::MissingGroupTypeError.new if type.nil?
-          fail Grape::Exceptions::UnsupportedGroupTypeError.new unless Grape::Validations::Types.group?(type)
+          raise Grape::Exceptions::MissingGroupTypeError.new if type.nil?
+          raise Grape::Exceptions::UnsupportedGroupTypeError.new unless Grape::Validations::Types.group?(type)
         end
 
         opts = attrs[1] || { type: Array }
@@ -158,7 +186,8 @@ module Grape
           parent:   self,
           optional: optional,
           type:     opts[:type],
-          &block)
+          &block
+        )
       end
 
       # Returns a new parameter scope, not nested under any current-level param
@@ -174,9 +203,24 @@ module Grape
           element:      nil,
           parent:       self,
           options:      @optional,
-          type:         Hash,
+          type:         type == Array ? Array : Hash,
           dependent_on: options[:dependent_on],
-          &block)
+          &block
+        )
+      end
+
+      # Returns a new parameter scope, subordinate to the current one and nested
+      # under the parameter corresponding to `attrs.first`.
+      # @param attrs [Array] the attributes passed to the `requires` or
+      #   `optional` invocation that opened this scope.
+      # @yield parameter scope
+      def new_group_scope(attrs, &block)
+        self.class.new(
+          api:          @api,
+          parent:       self,
+          group:        attrs.first,
+          &block
+        )
       end
 
       # Pushes declared params to parent or settings
@@ -187,7 +231,7 @@ module Grape
           @api.namespace_stackable(:declared_params, @declared_params)
 
           @api.route_setting(:declared_params, []) unless @api.route_setting(:declared_params)
-          @api.route_setting(:declared_params).concat @declared_params
+          @api.route_setting(:declared_params, @api.namespace_stackable(:declared_params).flatten)
         end
       end
 
@@ -204,25 +248,41 @@ module Grape
         default = validations[:default]
         doc_attrs[:default] = default if validations.key?(:default)
 
-        values = (options_key?(:values, :value, validations)) ? validations[:values][:value] : validations[:values]
+        if (values_hash = validations[:values]).is_a? Hash
+          values = values_hash[:value]
+          # NB: excepts is deprecated
+          excepts = values_hash[:except]
+        else
+          values = validations[:values]
+        end
         doc_attrs[:values] = values if values
 
-        coerce_type = guess_coerce_type(coerce_type, values)
+        except_values = options_key?(:except_values, :value, validations) ? validations[:except_values][:value] : validations[:except_values]
+
+        # NB. values and excepts should be nil, Proc, Array, or Range.
+        # Specifically, values should NOT be a Hash
+
+        # use values or excepts to guess coerce type when stated type is Array
+        coerce_type = guess_coerce_type(coerce_type, values, except_values, excepts)
 
         # default value should be present in values array, if both exist and are not procs
-        check_incompatible_option_values(values, default)
+        check_incompatible_option_values(default, values, except_values, excepts)
 
         # type should be compatible with values array, if both exist
-        validate_value_coercion(coerce_type, values)
+        validate_value_coercion(coerce_type, values, except_values, excepts)
 
         doc_attrs[:documentation] = validations.delete(:documentation) if validations.key?(:documentation)
 
         full_attrs = attrs.collect { |name| { name: name, full_name: full_name(name) } }
         @api.document_attribute(full_attrs, doc_attrs)
 
+        # slice out fail_fast attribute
+        opts = {}
+        opts[:fail_fast] = validations.delete(:fail_fast) || false
+
         # Validate for presence before any other validators
         if validations.key?(:presence) && validations[:presence]
-          validate('presence', validations[:presence], attrs, doc_attrs)
+          validate('presence', validations[:presence], attrs, doc_attrs, opts)
           validations.delete(:presence)
           validations.delete(:message) if validations.key?(:message)
         end
@@ -230,10 +290,10 @@ module Grape
         # Before we run the rest of the validators, let's handle
         # whatever coercion so that we are working with correctly
         # type casted values
-        coerce_type validations, attrs, doc_attrs
+        coerce_type validations, attrs, doc_attrs, opts
 
         validations.each do |type, options|
-          validate(type, options, attrs, doc_attrs)
+          validate(type, options, attrs, doc_attrs, opts)
         end
       end
 
@@ -252,7 +312,7 @@ module Grape
       # @raise [ArgumentError] if the given type options are invalid
       def infer_coercion(validations)
         if validations.key?(:type) && validations.key?(:types)
-          fail ArgumentError, ':type may not be supplied with :types'
+          raise ArgumentError, ':type may not be supplied with :types'
         end
 
         validations[:coerce] = (options_key?(:type, :value, validations) ? validations[:type][:value] : validations[:type]) if validations.key?(:type)
@@ -284,12 +344,12 @@ module Grape
       def check_coerce_with(validations)
         return unless validations.key?(:coerce_with)
         # type must be supplied for coerce_with..
-        fail ArgumentError, 'must supply type for coerce_with' unless validations.key?(:coerce)
+        raise ArgumentError, 'must supply type for coerce_with' unless validations.key?(:coerce)
 
         # but not special JSON types, which
         # already imply coercion method
         return unless [JSON, Array[JSON]].include? validations[:coerce]
-        fail ArgumentError, 'coerce_with disallowed for type: JSON'
+        raise ArgumentError, 'coerce_with disallowed for type: JSON'
       end
 
       # Add type coercion validation to this scope,
@@ -298,7 +358,7 @@ module Grape
       # composited from more than one +requires+/+optional+
       # parameter, and needs to be run before most other
       # validations.
-      def coerce_type(validations, attrs, doc_attrs)
+      def coerce_type(validations, attrs, doc_attrs, opts)
         check_coerce_with(validations)
 
         return unless validations.key?(:coerce)
@@ -308,57 +368,81 @@ module Grape
           method: validations[:coerce_with],
           message: validations[:coerce_message]
         }
-        validate('coerce', coerce_options, attrs, doc_attrs)
+        validate('coerce', coerce_options, attrs, doc_attrs, opts)
         validations.delete(:coerce_with)
         validations.delete(:coerce)
         validations.delete(:coerce_message)
       end
 
-      def guess_coerce_type(coerce_type, values)
-        return coerce_type if !values || values.is_a?(Proc)
-        return values.first.class if coerce_type == Array && (values.is_a?(Range) || !values.empty?)
+      def guess_coerce_type(coerce_type, *values_list)
+        return coerce_type unless coerce_type == Array
+        values_list.each do |values|
+          next if !values || values.is_a?(Proc)
+          return values.first.class if values.is_a?(Range) || !values.empty?
+        end
         coerce_type
       end
 
-      def check_incompatible_option_values(values, default)
-        return unless values && default
-        return if values.is_a?(Proc) || default.is_a?(Proc)
-        return if values.include?(default) || (Array(default) - Array(values)).empty?
-        fail Grape::Exceptions::IncompatibleOptionValues.new(:default, default, :values, values)
+      def check_incompatible_option_values(default, values, except_values, excepts)
+        return unless default && !default.is_a?(Proc)
+
+        if values && !values.is_a?(Proc)
+          raise Grape::Exceptions::IncompatibleOptionValues.new(:default, default, :values, values) \
+            unless Array(default).all? { |def_val| values.include?(def_val) }
+        end
+
+        if except_values && !except_values.is_a?(Proc)
+          raise Grape::Exceptions::IncompatibleOptionValues.new(:default, default, :except, except_values) \
+            unless Array(default).none? { |def_val| except_values.include?(def_val) }
+        end
+
+        return unless excepts && !excepts.is_a?(Proc)
+        raise Grape::Exceptions::IncompatibleOptionValues.new(:default, default, :except, excepts) \
+          unless Array(default).none? { |def_val| excepts.include?(def_val) }
       end
 
-      def validate(type, options, attrs, doc_attrs)
+      def validate(type, options, attrs, doc_attrs, opts)
         validator_class = Validations.validators[type.to_s]
 
-        if validator_class
-          value = validator_class.new(attrs, options, doc_attrs[:required], self)
-          @api.namespace_stackable(:validations, value)
-        else
-          fail Grape::Exceptions::UnknownValidator.new(type)
-        end
+        raise Grape::Exceptions::UnknownValidator.new(type) unless validator_class
+
+        factory = Grape::Validations::ValidatorFactory.new(attributes:      attrs,
+                                                           options:         options,
+                                                           required:        doc_attrs[:required],
+                                                           params_scope:    self,
+                                                           opts:            opts,
+                                                           validator_class: validator_class)
+        @api.namespace_stackable(:validations, factory)
       end
 
-      def validate_value_coercion(coerce_type, values)
-        return unless coerce_type && values
-        return if values.is_a?(Proc)
+      def validate_value_coercion(coerce_type, *values_list)
+        return unless coerce_type
         coerce_type = coerce_type.first if coerce_type.is_a?(Array)
-        value_types = values.is_a?(Range) ? [values.begin, values.end] : values
-        if coerce_type == Virtus::Attribute::Boolean
-          value_types = value_types.map { |type| Virtus::Attribute.build(type) }
+        values_list.each do |values|
+          next if !values || values.is_a?(Proc)
+          value_types = values.is_a?(Range) ? [values.begin, values.end] : values
+          if coerce_type == Virtus::Attribute::Boolean
+            value_types = value_types.map { |type| Virtus::Attribute.build(type) }
+          end
+          unless value_types.all? { |v| v.is_a? coerce_type }
+            raise Grape::Exceptions::IncompatibleOptionValues.new(:type, coerce_type, :values, values)
+          end
         end
-        return unless value_types.any? { |v| !v.is_a?(coerce_type) }
-        fail Grape::Exceptions::IncompatibleOptionValues.new(:type, coerce_type, :values, values)
       end
 
       def extract_message_option(attrs)
         return nil unless attrs.is_a?(Array)
         opts = attrs.last.is_a?(Hash) ? attrs.pop : {}
-        (opts.key?(:message) && !opts[:message].nil?) ? opts.delete(:message) : nil
+        opts.key?(:message) && !opts[:message].nil? ? opts.delete(:message) : nil
       end
 
       def options_key?(type, key, validations)
         validations[type].respond_to?(:key?) && validations[type].key?(key) && !validations[type][key].nil?
       end
+
+      def all_element_blank?(parameters)
+        params(parameters).respond_to?(:all?) && params(parameters).all?(&:blank?)
+      end
     end
   end
 end
