@@ -10,14 +10,16 @@ module Grape
         @converter = Types.build_coercer(type, @option[:method])
       end
 
+      def validate(request)
+        super
+      end
+
       def validate_param!(attr_name, params)
-        fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message(:coerce) unless params.is_a? Hash
+        raise Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message(:coerce) unless params.is_a? Hash
+        return unless requires_coercion?(params[attr_name])
         new_value = coerce_value(params[attr_name])
-        if valid_type?(new_value)
-          params[attr_name] = new_value
-        else
-          fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message(:coerce)
-        end
+        raise Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message(:coerce) unless valid_type?(new_value)
+        params[attr_name] = new_value
       end
 
       private
@@ -62,6 +64,11 @@ module Grape
       def type
         @option[:type].is_a?(Hash) ? @option[:type][:value] : @option[:type]
       end
+
+      def requires_coercion?(value)
+        # JSON types do not require coercion if value is valid
+        !valid_type?(value) || converter.coercer.respond_to?(:method) && !converter.is_a?(Grape::Validations::Types::Json)
+      end
     end
   end
 end
