@@ -215,7 +215,7 @@ describe Grape::Endpoint do
         [cookie.name, cookie]
       end]
       expect(cookies.size).to eq(2)
-      %w(and_this delete_this_cookie).each do |cookie_name|
+      %w[and_this delete_this_cookie].each do |cookie_name|
         cookie = cookies[cookie_name]
         expect(cookie).not_to be_nil
         expect(cookie.value).to eq('deleted')
@@ -239,7 +239,7 @@ describe Grape::Endpoint do
         [cookie.name, cookie]
       end]
       expect(cookies.size).to eq(2)
-      %w(and_this delete_this_cookie).each do |cookie_name|
+      %w[and_this delete_this_cookie].each do |cookie_name|
         cookie = cookies[cookie_name]
         expect(cookie).not_to be_nil
         expect(cookie.value).to eq('deleted')
@@ -249,18 +249,109 @@ describe Grape::Endpoint do
     end
   end
 
+  describe '#params' do
+    context 'default class' do
+      it 'should be a ActiveSupport::HashWithIndifferentAccess' do
+        subject.get '/foo' do
+          params.class
+        end
+
+        get '/foo'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('ActiveSupport::HashWithIndifferentAccess')
+      end
+    end
+
+    context 'sets a value to params' do
+      it 'params' do
+        subject.params do
+          requires :a, type: String
+        end
+        subject.get '/foo' do
+          params[:a] = 'bar'
+        end
+
+        get '/foo', a: 'foo'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('bar')
+      end
+    end
+  end
+
   describe '#declared' do
     before do
+      subject.format :json
       subject.params do
         requires :first
         optional :second
         optional :third, default: 'third-default'
         optional :nested, type: Hash do
           optional :fourth
+          optional :fifth
+          optional :nested_two, type: Hash do
+            optional :sixth
+            optional :nested_three, type: Hash do
+              optional :seventh
+            end
+          end
+        end
+        optional :nested_arr, type: Array do
+          optional :eighth
         end
       end
     end
 
+    context 'when params are not built with default class' do
+      it 'returns an object that corresponds with the params class - hash with indifferent access' do
+        subject.params do
+          build_with Grape::Extensions::ActiveSupport::HashWithIndifferentAccess::ParamBuilder
+        end
+        subject.get '/declared' do
+          d = declared(params, include_missing: true)
+          { declared_class: d.class.to_s }
+        end
+
+        get '/declared?first=present'
+        expect(JSON.parse(last_response.body)['declared_class']).to eq('ActiveSupport::HashWithIndifferentAccess')
+      end
+
+      it 'returns an object that corresponds with the params class - hashie mash' do
+        subject.params do
+          build_with Grape::Extensions::Hashie::Mash::ParamBuilder
+        end
+        subject.get '/declared' do
+          d = declared(params, include_missing: true)
+          { declared_class: d.class.to_s }
+        end
+
+        get '/declared?first=present'
+        expect(JSON.parse(last_response.body)['declared_class']).to eq('Hashie::Mash')
+      end
+
+      it 'returns an object that corresponds with the params class - hash' do
+        subject.params do
+          build_with Grape::Extensions::Hash::ParamBuilder
+        end
+        subject.get '/declared' do
+          d = declared(params, include_missing: true)
+          { declared_class: d.class.to_s }
+        end
+
+        get '/declared?first=present'
+        expect(JSON.parse(last_response.body)['declared_class']).to eq('Hash')
+      end
+    end
+
+    it 'should show nil for nested params if include_missing is true' do
+      subject.get '/declared' do
+        declared(params, include_missing: true)
+      end
+
+      get '/declared?first=present'
+      expect(last_response.status).to eq(200)
+      expect(JSON.parse(last_response.body)['nested']['fourth']).to be_nil
+    end
+
     it 'does not work in a before filter' do
       subject.before do
         declared(params)
@@ -268,41 +359,36 @@ describe Grape::Endpoint do
       subject.get('/declared') { declared(params) }
 
       expect { get('/declared') }.to raise_error(
-        Grape::DSL::InsideRoute::MethodNotYetAvailable)
+        Grape::DSL::InsideRoute::MethodNotYetAvailable
+      )
     end
 
     it 'has as many keys as there are declared params' do
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params).keys
-        ''
+        declared(params)
       end
       get '/declared?first=present'
       expect(last_response.status).to eq(200)
-      expect(inner_params.size).to eq(4)
+      expect(JSON.parse(last_response.body).keys.size).to eq(5)
     end
 
     it 'has a optional param with default value all the time' do
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params)
-        ''
+        declared(params)
       end
       get '/declared?first=one'
       expect(last_response.status).to eq(200)
-      expect(inner_params[:third]).to eql('third-default')
+      expect(JSON.parse(last_response.body)['third']).to eql('third-default')
     end
 
     it 'builds nested params' do
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params)
-        ''
+        declared(params)
       end
 
       get '/declared?first=present&nested[fourth]=1'
       expect(last_response.status).to eq(200)
-      expect(inner_params[:nested].keys.size).to eq 1
+      expect(JSON.parse(last_response.body)['nested'].keys.size).to eq 3
     end
 
     it 'builds nested params when given array' do
@@ -316,76 +402,64 @@ describe Grape::Endpoint do
           optional :fourth
         end
       end
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params)
-        ''
+        declared(params)
       end
 
       get '/declared?first=present&nested[][fourth]=1&nested[][fourth]=2'
       expect(last_response.status).to eq(200)
-      expect(inner_params[:nested].size).to eq 2
+      expect(JSON.parse(last_response.body)['nested'].size).to eq 2
     end
 
-    it 'builds nested params' do
-      inner_params = nil
-      subject.get '/declared' do
-        inner_params = declared(params)
-        ''
-      end
-
-      get '/declared?first=present&nested[fourth]=1'
-      expect(last_response.status).to eq(200)
-      expect(inner_params[:nested].keys.size).to eq 1
-    end
-
-    context 'sets nested array when the param is missing' do
-      it 'to be array when include_missing is true' do
-        inner_params = nil
+    context 'sets nested objects when the param is missing' do
+      it 'to be a hash when include_missing is true' do
         subject.get '/declared' do
-          inner_params = declared(params, include_missing: true)
-          ''
+          declared(params, include_missing: true)
         end
 
         get '/declared?first=present'
         expect(last_response.status).to eq(200)
-        expect(inner_params[:nested]).to be_a(Array)
+        expect(JSON.parse(last_response.body)['nested']).to be_a(Hash)
+      end
+
+      it 'to be an array when include_missing is true' do
+        subject.get '/declared' do
+          declared(params, include_missing: true)
+        end
+
+        get '/declared?first=present'
+        expect(last_response.status).to eq(200)
+        expect(JSON.parse(last_response.body)['nested_arr']).to be_a(Array)
       end
 
       it 'to be nil when include_missing is false' do
-        inner_params = nil
         subject.get '/declared' do
-          inner_params = declared(params, include_missing: false)
-          ''
+          declared(params, include_missing: false)
         end
 
         get '/declared?first=present'
         expect(last_response.status).to eq(200)
-        expect(inner_params[:nested]).to be_nil
+        expect(JSON.parse(last_response.body)['nested']).to be_nil
       end
     end
 
     it 'filters out any additional params that are given' do
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params)
-        ''
+        declared(params)
       end
       get '/declared?first=one&other=two'
       expect(last_response.status).to eq(200)
-      expect(inner_params.key?(:other)).to eq false
+      expect(JSON.parse(last_response.body).key?(:other)).to eq false
     end
 
     it 'stringifies if that option is passed' do
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params, stringify: true)
-        ''
+        declared(params, stringify: true)
       end
 
       get '/declared?first=one&other=two'
       expect(last_response.status).to eq(200)
-      expect(inner_params['first']).to eq 'one'
+      expect(JSON.parse(last_response.body)['first']).to eq 'one'
     end
 
     it 'does not include missing attributes if that option is passed' do
@@ -409,7 +483,7 @@ describe Grape::Endpoint do
         ''
       end
 
-      post '/declared', MultiJson.dump(first: 'one', boolean: false), 'CONTENT_TYPE' => 'application/json'
+      post '/declared', ::Grape::Json.dump(first: 'one', boolean: false), 'CONTENT_TYPE' => 'application/json'
       expect(last_response.status).to eq(201)
     end
 
@@ -424,11 +498,11 @@ describe Grape::Endpoint do
         ''
       end
 
-      post '/declared', MultiJson.dump(first: 'one', second: nil), 'CONTENT_TYPE' => 'application/json'
+      post '/declared', ::Grape::Json.dump(first: 'one', second: nil), 'CONTENT_TYPE' => 'application/json'
       expect(last_response.status).to eq(201)
     end
 
-    it 'does not include missing attributes when there are nested hashes' do
+    it 'includes missing attributes with defaults when there are nested hashes' do
       subject.get '/dummy' do
       end
 
@@ -446,69 +520,269 @@ describe Grape::Endpoint do
         end
       end
 
-      inner_params = nil
       subject.get '/declared' do
-        inner_params = declared(params, include_missing: false)
-        ''
+        declared(params, include_missing: false)
       end
 
       get '/declared?first=present&nested[fourth]=&nested[nested_nested][sixth]=sixth'
-
+      json = JSON.parse(last_response.body)
       expect(last_response.status).to eq(200)
-      expect(inner_params[:first]).to eq 'present'
-      expect(inner_params[:nested].keys).to eq %w(fourth fifth nested_nested)
-      expect(inner_params[:nested][:fourth]).to eq ''
-      expect(inner_params[:nested][:nested_nested].keys).to eq %w(sixth seven)
-      expect(inner_params[:nested][:nested_nested][:sixth]).to eq 'sixth'
+      expect(json['first']).to eq 'present'
+      expect(json['nested'].keys).to eq %w[fourth fifth nested_nested]
+      expect(json['nested']['fourth']).to eq ''
+      expect(json['nested']['nested_nested'].keys).to eq %w[sixth seven]
+      expect(json['nested']['nested_nested']['sixth']).to eq 'sixth'
+    end
+
+    it 'does not include missing attributes when there are nested hashes' do
+      subject.get '/dummy' do
+      end
+
+      subject.params do
+        requires :first
+        optional :second
+        optional :third
+        optional :nested, type: Hash do
+          optional :fourth
+          optional :fifth
+        end
+      end
+
+      subject.get '/declared' do
+        declared(params, include_missing: false)
+      end
+
+      get '/declared?first=present&nested[fourth]=4'
+      json = JSON.parse(last_response.body)
+      expect(last_response.status).to eq(200)
+      expect(json['first']).to eq 'present'
+      expect(json['nested'].keys).to eq %w[fourth]
+      expect(json['nested']['fourth']).to eq '4'
     end
   end
 
   describe '#declared; call from child namespace' do
     before do
       subject.format :json
-      subject.namespace :something do
+      subject.namespace :parent do
         params do
-          requires :id, type: Integer
+          requires :parent_name, type: String
         end
-        resource ':id' do
+
+        namespace ':parent_name' do
           params do
-            requires :foo
-            optional :bar
+            requires :child_name, type: String
+            requires :child_age, type: Integer
           end
+
+          namespace ':child_name' do
+            params do
+              requires :grandchild_name, type: String
+            end
+
+            get ':grandchild_name' do
+              {
+                'params' => params,
+                'without_parent_namespaces' => declared(params, include_parent_namespaces: false),
+                'with_parent_namespaces' => declared(params, include_parent_namespaces: true)
+              }
+            end
+          end
+        end
+      end
+
+      get '/parent/foo/bar/baz', child_age: 5, extra: 'hello'
+    end
+
+    let(:parsed_response) { JSON.parse(last_response.body, symbolize_names: true) }
+
+    it { expect(last_response.status).to eq 200 }
+
+    context 'with include_parent_namespaces: false' do
+      it 'returns declared parameters only from current namespace' do
+        expect(parsed_response[:without_parent_namespaces]).to eq(
+          grandchild_name: 'baz'
+        )
+      end
+    end
+
+    context 'with include_parent_namespaces: true' do
+      it 'returns declared parameters from every parent namespace' do
+        expect(parsed_response[:with_parent_namespaces]).to eq(
+          parent_name: 'foo',
+          child_name: 'bar',
+          grandchild_name: 'baz',
+          child_age: 5
+        )
+      end
+    end
+
+    context 'without declaration' do
+      it 'returns all requested parameters' do
+        expect(parsed_response[:params]).to eq(
+          parent_name: 'foo',
+          child_name: 'bar',
+          grandchild_name: 'baz',
+          child_age: 5,
+          extra: 'hello'
+        )
+      end
+    end
+  end
+
+  describe '#declared; from a nested mounted endpoint' do
+    before do
+      doubly_mounted = Class.new(Grape::API)
+      doubly_mounted.namespace :more do
+        params do
+          requires :y, type: Integer
+        end
+        route_param :y do
           get do
             {
               params: params,
               declared_params: declared(params)
             }
           end
-          params do
-            requires :happy
-            optional :days
+        end
+      end
+
+      mounted = Class.new(Grape::API)
+      mounted.namespace :another do
+        params do
+          requires :mount_space, type: Integer
+        end
+        route_param :mount_space do
+          mount doubly_mounted
+        end
+      end
+
+      subject.format :json
+      subject.namespace :something do
+        params do
+          requires :id, type: Integer
+        end
+        resource ':id' do
+          mount mounted
+        end
+      end
+    end
+
+    it 'can access parent attributes' do
+      get '/something/123/another/456/more/789'
+      expect(last_response.status).to eq 200
+      json = JSON.parse(last_response.body, symbolize_names: true)
+
+      # test all three levels of params
+      expect(json[:declared_params][:y]).to eq 789
+      expect(json[:declared_params][:mount_space]).to eq 456
+      expect(json[:declared_params][:id]).to eq 123
+    end
+  end
+
+  describe '#declared; mixed nesting' do
+    before do
+      subject.format :json
+      subject.resource :users do
+        route_param :id, type: Integer, desc: 'ID desc' do
+          # Adding this causes route_setting(:declared_params) to be nil for the
+          # get block in namespace 'foo' below
+          get do
           end
-          get '/test' do
-            {
-              params: params,
-              declared_params: declared(params, include_parent_namespaces: false)
-            }
+
+          namespace 'foo' do
+            get do
+              {
+                params: params,
+                declared_params: declared(params),
+                declared_params_no_parent: declared(params, include_parent_namespaces: false)
+              }
+            end
           end
         end
       end
     end
 
-    it 'should include params defined in the parent namespace' do
-      get '/something/123', foo: 'test', extra: 'hello'
+    it 'can access parent route_param' do
+      get '/users/123/foo', bar: 'bar'
       expect(last_response.status).to eq 200
       json = JSON.parse(last_response.body, symbolize_names: true)
-      expect(json[:params][:id]).to eq 123
-      expect(json[:declared_params].keys).to match_array [:foo, :bar, :id]
+
+      expect(json[:declared_params][:id]).to eq 123
+      expect(json[:declared_params_no_parent][:id]).to eq nil
+    end
+  end
+
+  describe '#declared; with multiple route_param' do
+    before do
+      mounted = Class.new(Grape::API)
+      mounted.namespace :albums do
+        get do
+          declared(params)
+        end
+      end
+
+      subject.format :json
+      subject.namespace :artists do
+        route_param :id, type: Integer do
+          get do
+            declared(params)
+          end
+
+          params do
+            requires :filter, type: String
+          end
+          get :some_route do
+            declared(params)
+          end
+        end
+
+        route_param :artist_id, type: Integer do
+          namespace :compositions do
+            get do
+              declared(params)
+            end
+          end
+        end
+
+        route_param :compositor_id, type: Integer do
+          mount mounted
+        end
+      end
     end
 
-    it 'does not include params defined in the parent namespace with include_parent_namespaces: false' do
-      get '/something/123/test', happy: 'test', extra: 'hello'
-      expect(last_response.status).to eq 200
+    it 'return only :id without :artist_id' do
+      get '/artists/1'
       json = JSON.parse(last_response.body, symbolize_names: true)
-      expect(json[:params][:id]).to eq 123
-      expect(json[:declared_params].keys).to match_array [:happy, :days]
+
+      expect(json.key?(:id)).to be_truthy
+      expect(json.key?(:artist_id)).not_to be_truthy
+    end
+
+    it 'return only :artist_id without :id' do
+      get '/artists/1/compositions'
+      json = JSON.parse(last_response.body, symbolize_names: true)
+
+      expect(json.key?(:artist_id)).to be_truthy
+      expect(json.key?(:id)).not_to be_truthy
+    end
+
+    it 'return :filter and :id parameters in declared for second enpoint inside route_param' do
+      get '/artists/1/some_route', filter: 'some_filter'
+      json = JSON.parse(last_response.body, symbolize_names: true)
+
+      expect(json.key?(:filter)).to be_truthy
+      expect(json.key?(:id)).to be_truthy
+      expect(json.key?(:artist_id)).not_to be_truthy
+    end
+
+    it 'return :compositor_id for mounter in route_param' do
+      get '/artists/1/albums'
+      json = JSON.parse(last_response.body, symbolize_names: true)
+
+      expect(json.key?(:compositor_id)).to be_truthy
+      expect(json.key?(:id)).not_to be_truthy
+      expect(json.key?(:artist_id)).not_to be_truthy
     end
   end
 
@@ -577,13 +851,14 @@ describe Grape::Endpoint do
               params[:person_email]
             end
 
-            namespace :inner, requirements: { number: /[0-9]/, person_email: /someone@(.*).com/ }do
+            namespace :inner, requirements: { number: /[0-9]/, person_email: /someone@(.*).com/ } do
               get '/:person_email/test/:number' do
                 params[:person_email] << params[:number]
               end
             end
           end
         end
+
         it 'parse email param with provided requirements for params' do
           get '/outer/abc@example.com'
           expect(last_response.body).to eq('abc@example.com')
@@ -611,7 +886,7 @@ describe Grape::Endpoint do
       end
 
       it 'converts JSON bodies to params' do
-        post '/request_body', MultiJson.dump(user: 'Bobby T.'), 'CONTENT_TYPE' => 'application/json'
+        post '/request_body', ::Grape::Json.dump(user: 'Bobby T.'), 'CONTENT_TYPE' => 'application/json'
         expect(last_response.body).to eq('Bobby T.')
       end
 
@@ -620,14 +895,26 @@ describe Grape::Endpoint do
         expect(last_response.body).to eq('')
       end
 
-      it 'converts XML bodies to params' do
-        post '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
-        expect(last_response.body).to eq('Bobby T.')
-      end
+      if Object.const_defined? :MultiXml
+        it 'converts XML bodies to params' do
+          post '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
+          expect(last_response.body).to eq('Bobby T.')
+        end
 
-      it 'converts XML bodies to params' do
-        put '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
-        expect(last_response.body).to eq('Bobby T.')
+        it 'converts XML bodies to params' do
+          put '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
+          expect(last_response.body).to eq('Bobby T.')
+        end
+      else
+        it 'converts XML bodies to params' do
+          post '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
+          expect(last_response.body).to eq('{"__content__"=>"Bobby T."}')
+        end
+
+        it 'converts XML bodies to params' do
+          put '/request_body', '<user>Bobby T.</user>', 'CONTENT_TYPE' => 'application/xml'
+          expect(last_response.body).to eq('{"__content__"=>"Bobby T."}')
+        end
       end
 
       it 'does not include parameters not defined by the body' do
@@ -635,7 +922,7 @@ describe Grape::Endpoint do
           error! 400, 'expected nil' if params[:version]
           params[:user]
         end
-        post '/omitted_params', MultiJson.dump(user: 'Bob'), 'CONTENT_TYPE' => 'application/json'
+        post '/omitted_params', ::Grape::Json.dump(user: 'Bob'), 'CONTENT_TYPE' => 'application/json'
         expect(last_response.status).to eq(201)
         expect(last_response.body).to eq('Bob')
       end
@@ -652,6 +939,18 @@ describe Grape::Endpoint do
       expect(last_response.body).to eq('{"error":"The requested content-type \'application/xml\' is not supported."}')
     end
 
+    it 'does not accept text/plain in JSON format if application/json is specified as content type' do
+      subject.format :json
+      subject.default_format :json
+      subject.put '/request_body' do
+        params[:user]
+      end
+      put '/request_body', ::Grape::Json.dump(user: 'Bob'), 'CONTENT_TYPE' => 'text/plain'
+
+      expect(last_response.status).to eq(406)
+      expect(last_response.body).to eq('{"error":"The requested content-type \'text/plain\' is not supported."}')
+    end
+
     context 'content type with params' do
       before do
         subject.format :json
@@ -660,7 +959,7 @@ describe Grape::Endpoint do
         subject.post do
           params[:data]
         end
-        post '/', MultiJson.dump(data: { some: 'payload' }), 'CONTENT_TYPE' => 'application/json'
+        post '/', ::Grape::Json.dump(data: { some: 'payload' }), 'CONTENT_TYPE' => 'application/json'
       end
 
       it 'should not response with 406 for same type without params' do
@@ -708,6 +1007,21 @@ describe Grape::Endpoint do
         expect(JSON.parse(last_response.body)['params']).to eq '123'
       end
     end
+
+    context 'sets a value to params' do
+      it 'params' do
+        subject.params do
+          requires :a, type: String
+        end
+        subject.get '/foo' do
+          params[:a] = 'bar'
+        end
+
+        get '/foo', a: 'foo'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('bar')
+      end
+    end
   end
 
   describe '#error!' do
@@ -910,7 +1224,7 @@ describe Grape::Endpoint do
 
       get '/all_filters'
       json = JSON.parse(last_response.body)
-      expect(json.keys).to match_array %w(before before_validation after_validation endpoint after)
+      expect(json.keys).to match_array %w[before before_validation after_validation endpoint after]
     end
 
     context 'when terminating the response with error!' do
@@ -927,7 +1241,7 @@ describe Grape::Endpoint do
 
         get '/error_filters'
         expect(last_response.status).to eql 500
-        expect(called).to match_array %w(before before_validation)
+        expect(called).to match_array %w[before before_validation]
       end
 
       it 'allows prior and parent filters of same type to run' do
@@ -945,38 +1259,98 @@ describe Grape::Endpoint do
 
         get '/parent/hello'
         expect(last_response.status).to eql 500
-        expect(called).to match_array %w(parent prior)
+        expect(called).to match_array %w[parent prior]
       end
     end
   end
 
   context 'anchoring' do
-    verbs = %w(post get head delete put options patch)
-
-    verbs.each do |verb|
-      it "allows for the anchoring option with a #{verb.upcase} method" do
-        subject.send(verb, '/example', anchor: true) do
-          verb
-        end
-        send(verb, '/example/and/some/more')
+    describe 'delete 204' do
+      it 'allows for the anchoring option with a delete method' do
+        subject.send(:delete, '/example', anchor: true) {}
+        send(:delete, '/example/and/some/more')
         expect(last_response.status).to eql 404
       end
 
-      it "anchors paths by default for the #{verb.upcase} method" do
-        subject.send(verb, '/example') do
-          verb
-        end
-        send(verb, '/example/and/some/more')
+      it 'anchors paths by default for the delete method' do
+        subject.send(:delete, '/example') {}
+        send(:delete, '/example/and/some/more')
         expect(last_response.status).to eql 404
       end
 
-      it "responds to /example/and/some/more for the non-anchored #{verb.upcase} method" do
-        subject.send(verb, '/example', anchor: false) do
-          verb
+      it 'responds to /example/and/some/more for the non-anchored delete method' do
+        subject.send(:delete, '/example', anchor: false) {}
+        send(:delete, '/example/and/some/more')
+        expect(last_response.status).to eql 204
+        expect(last_response.body).to be_empty
+      end
+    end
+
+    describe 'delete 200, with response body' do
+      it 'responds to /example/and/some/more for the non-anchored delete method' do
+        subject.send(:delete, '/example', anchor: false) do
+          status 200
+          body 'deleted'
+        end
+        send(:delete, '/example/and/some/more')
+        expect(last_response.status).to eql 200
+        expect(last_response.body).not_to be_empty
+      end
+    end
+
+    describe 'delete 200, with a return value (no explicit body)' do
+      it 'responds to /example delete method' do
+        subject.delete(:example) { 'deleted' }
+        delete '/example'
+        expect(last_response.status).to eql 200
+        expect(last_response.body).not_to be_empty
+      end
+    end
+
+    describe 'delete 204, with nil has return value (no explicit body)' do
+      it 'responds to /example delete method' do
+        subject.delete(:example) { nil }
+        delete '/example'
+        expect(last_response.status).to eql 204
+        expect(last_response.body).to be_empty
+      end
+    end
+
+    describe 'delete 204, with empty array has return value (no explicit body)' do
+      it 'responds to /example delete method' do
+        subject.delete(:example) { '' }
+        delete '/example'
+        expect(last_response.status).to eql 204
+        expect(last_response.body).to be_empty
+      end
+    end
+
+    describe 'all other' do
+      %w[post get head put options patch].each do |verb|
+        it "allows for the anchoring option with a #{verb.upcase} method" do
+          subject.send(verb, '/example', anchor: true) do
+            verb
+          end
+          send(verb, '/example/and/some/more')
+          expect(last_response.status).to eql 404
+        end
+
+        it "anchors paths by default for the #{verb.upcase} method" do
+          subject.send(verb, '/example') do
+            verb
+          end
+          send(verb, '/example/and/some/more')
+          expect(last_response.status).to eql 404
+        end
+
+        it "responds to /example/and/some/more for the non-anchored #{verb.upcase} method" do
+          subject.send(verb, '/example', anchor: false) do
+            verb
+          end
+          send(verb, '/example/and/some/more')
+          expect(last_response.status).to eql verb == 'post' ? 201 : 200
+          expect(last_response.body).to eql verb == 'head' ? '' : verb
         end
-        send(verb, '/example/and/some/more')
-        expect(last_response.status).to eql verb == 'post' ? 201 : 200
-        expect(last_response.body).to eql verb == 'head' ? '' : verb
       end
     end
   end
@@ -1095,6 +1469,9 @@ describe Grape::Endpoint do
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :before_validation }),
+        have_attributes(name: 'endpoint_run_validators.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                                          validators: [],
+                                                                          request: a_kind_of(Grape::Request) }),
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after_validation }),
@@ -1116,6 +1493,9 @@ describe Grape::Endpoint do
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :before_validation }),
+        have_attributes(name: 'endpoint_run_validators.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
+                                                                          validators: [],
+                                                                          request: a_kind_of(Grape::Request) }),
         have_attributes(name: 'endpoint_run_filters.grape', payload: { endpoint: a_kind_of(Grape::Endpoint),
                                                                        filters: [],
                                                                        type: :after_validation }),
