@@ -6,11 +6,11 @@ describe Grape::Exceptions::ValidationErrors do
   let(:validation_error) { OpenStruct.new(params: [validation_message]) }
 
   context 'initialize' do
-    let(:headers) {
+    let(:headers) do
       {
         'A-Header-Key' => 'A-Header-Value'
       }
-    }
+    end
 
     subject do
       described_class.new(errors: [validation_error], headers: headers)
@@ -69,9 +69,9 @@ describe Grape::Exceptions::ValidationErrors do
       get '/exactly_one_of', beer: 'string', wine: 'anotherstring'
       expect(last_response.status).to eq(400)
       expect(JSON.parse(last_response.body)).to eq([
-        'params' => %w(beer wine),
-        'messages' => ['are mutually exclusive']
-      ])
+                                                     'params' => %w[beer wine],
+                                                     'messages' => ['are mutually exclusive']
+                                                   ])
     end
   end
 end
