@@ -4,46 +4,53 @@ describe Grape::Middleware::Formatter do
   subject { Grape::Middleware::Formatter.new(app) }
   before { allow(subject).to receive(:dup).and_return(subject) }
 
-  let(:app) { ->(_env) { [200, {}, [@body || { 'foo' => 'bar' }]] } }
+  let(:body) { { 'foo' => 'bar' } }
+  let(:app) { ->(_env) { [200, {}, [body]] } }
 
   context 'serialization' do
+    let(:body) { { 'abc' => 'def' } }
     it 'looks at the bodies for possibly serializable data' do
-      @body = { 'abc' => 'def' }
       _, _, bodies = *subject.call('PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json')
-      bodies.each { |b| expect(b).to eq(MultiJson.dump(@body)) }
+      bodies.each { |b| expect(b).to eq(::Grape::Json.dump(body)) }
     end
 
-    it 'calls #to_json since default format is json' do
-      @body = ['foo']
-      @body.instance_eval do
-        def to_json
-          "\"bar\""
+    context 'default format' do
+      let(:body) { ['foo'] }
+      it 'calls #to_json since default format is json' do
+        body.instance_eval do
+          def to_json
+            '"bar"'
+          end
         end
-      end
 
-      subject.call('PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json').to_a.last.each { |b| expect(b).to eq('"bar"') }
+        subject.call('PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json').to_a.last.each { |b| expect(b).to eq('"bar"') }
+      end
     end
 
-    it 'calls #to_json if the content type is jsonapi' do
-      @body = { 'foos' => [{ 'bar' => 'baz' }] }
-      @body.instance_eval do
-        def to_json
-          "{\"foos\":[{\"bar\":\"baz\"}] }"
+    context 'jsonapi' do
+      let(:body) { { 'foos' => [{ 'bar' => 'baz' }] } }
+      it 'calls #to_json if the content type is jsonapi' do
+        body.instance_eval do
+          def to_json
+            '{"foos":[{"bar":"baz"}] }'
+          end
         end
-      end
 
-      subject.call('PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/vnd.api+json').to_a.last.each { |b| expect(b).to eq('{"foos":[{"bar":"baz"}] }') }
+        subject.call('PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/vnd.api+json').to_a.last.each { |b| expect(b).to eq('{"foos":[{"bar":"baz"}] }') }
+      end
     end
 
-    it 'calls #to_xml if the content type is xml' do
-      @body = 'string'
-      @body.instance_eval do
-        def to_xml
-          '<bar/>'
+    context 'xml' do
+      let(:body) { 'string' }
+      it 'calls #to_xml if the content type is xml' do
+        body.instance_eval do
+          def to_xml
+            '<bar/>'
+          end
         end
-      end
 
-      subject.call('PATH_INFO' => '/somewhere.xml', 'HTTP_ACCEPT' => 'application/json').to_a.last.each { |b| expect(b).to eq('<bar/>') }
+        subject.call('PATH_INFO' => '/somewhere.xml', 'HTTP_ACCEPT' => 'application/json').to_a.last.each { |b| expect(b).to eq('<bar/>') }
+      end
     end
   end
 
@@ -54,7 +61,7 @@ describe Grape::Middleware::Formatter do
     end
 
     it 'rescues formatter-specific exceptions' do
-      allow(formatter).to receive(:call) { fail Grape::Exceptions::InvalidFormatter.new(String, 'xml') }
+      allow(formatter).to receive(:call) { raise Grape::Exceptions::InvalidFormatter.new(String, 'xml') }
 
       expect do
         catch(:error) { subject.call('PATH_INFO' => '/somewhere.xml', 'HTTP_ACCEPT' => 'application/json') }
@@ -62,7 +69,7 @@ describe Grape::Middleware::Formatter do
     end
 
     it 'does not rescue other exceptions' do
-      allow(formatter).to receive(:call) { fail StandardError }
+      allow(formatter).to receive(:call) { raise StandardError }
 
       expect do
         catch(:error) { subject.call('PATH_INFO' => '/somewhere.xml', 'HTTP_ACCEPT' => 'application/json') }
@@ -189,10 +196,12 @@ describe Grape::Middleware::Formatter do
       _, _, body = subject.call('PATH_INFO' => '/info.custom')
       expect(body.body).to eq(['CUSTOM FORMAT'])
     end
-    it 'uses default json formatter' do
-      @body = ['blah']
-      _, _, body = subject.call('PATH_INFO' => '/info.json')
-      expect(body.body).to eq(['["blah"]'])
+    context 'default' do
+      let(:body) { ['blah'] }
+      it 'uses default json formatter' do
+        _, _, body = subject.call('PATH_INFO' => '/info.json')
+        expect(body.body).to eq(['["blah"]'])
+      end
     end
     it 'uses custom json formatter' do
       subject.options[:formatters][:json] = ->(_obj, _env) { 'CUSTOM JSON FORMAT' }
@@ -214,7 +223,7 @@ describe Grape::Middleware::Formatter do
   end
 
   context 'input' do
-    %w(POST PATCH PUT DELETE).each do |method|
+    %w[POST PATCH PUT DELETE].each do |method|
       ['application/json', 'application/json; charset=utf-8'].each do |content_type|
         context content_type do
           it "parses the body from #{method} and copies values into rack.request.form_hash" do
@@ -265,7 +274,11 @@ describe Grape::Middleware::Formatter do
           'rack.input' => io,
           'CONTENT_LENGTH' => io.length
         )
-        expect(subject.env['rack.request.form_hash']['thing']['name']).to eq('Test')
+        if Object.const_defined? :MultiXml
+          expect(subject.env['rack.request.form_hash']['thing']['name']).to eq('Test')
+        else
+          expect(subject.env['rack.request.form_hash']['thing']['name']['__content__']).to eq('Test')
+        end
       end
       [Rack::Request::FORM_DATA_MEDIA_TYPES, Rack::Request::PARSEABLE_DATA_MEDIA_TYPES].flatten.each do |content_type|
         it "ignores #{content_type}" do
@@ -284,10 +297,10 @@ describe Grape::Middleware::Formatter do
   end
 
   context 'send file' do
-    let(:app) { ->(_env) { [200, {}, @body] } }
+    let(:body) { Grape::ServeFile::FileResponse.new('file') }
+    let(:app) { ->(_env) { [200, {}, body] } }
 
     it 'returns Grape::Uril::SendFileReponse' do
-      @body = Grape::ServeFile::FileResponse.new('file')
       env = { 'PATH_INFO' => '/somewhere', 'HTTP_ACCEPT' => 'application/json' }
       expect(subject.call(env)).to be_a(Grape::ServeFile::SendfileResponse)
     end
@@ -311,4 +324,28 @@ describe Grape::Middleware::Formatter do
       expect(bodies.body.first).to eq({ message: 'invalid' }.to_json)
     end
   end
+
+  context 'custom parser raises exception and rescue options are enabled for backtrace and original_exception' do
+    it 'adds the backtrace and original_exception to the error output' do
+      subject = Grape::Middleware::Formatter.new(
+        app,
+        rescue_options: { backtrace: true, original_exception: true },
+        parsers: { json: ->(_object, _env) { raise StandardError, 'fail' } }
+      )
+      io = StringIO.new('{invalid}')
+      error = catch(:error) {
+        subject.call(
+          'PATH_INFO' => '/info',
+          'REQUEST_METHOD' => 'POST',
+          'CONTENT_TYPE' => 'application/json',
+          'rack.input' => io,
+          'CONTENT_LENGTH' => io.length
+        )
+      }
+
+      expect(error[:message]).to eq 'fail'
+      expect(error[:backtrace].size).to be >= 1
+      expect(error[:original_exception].class).to eq StandardError
+    end
+  end
 end
