@@ -9,7 +9,12 @@ describe Grape::Validations::AllowBlankValidator do
         params do
           requires :name, allow_blank: false
         end
-        get
+        get '/disallow_blank'
+
+        params do
+          optional :name, type: String, allow_blank: false
+        end
+        get '/opt_disallow_string_blank'
 
         params do
           optional :name, allow_blank: false
@@ -52,9 +57,9 @@ describe Grape::Validations::AllowBlankValidator do
         get '/allow_float_blank'
 
         params do
-          requires :val, type: Fixnum, allow_blank: true
+          requires :val, type: Integer, allow_blank: true
         end
-        get '/allow_fixnum_blank'
+        get '/allow_integer_blank'
 
         params do
           requires :val, type: Symbol, allow_blank: true
@@ -168,9 +173,9 @@ describe Grape::Validations::AllowBlankValidator do
           get '/allow_float_blank'
 
           params do
-            requires :val, type: Fixnum, allow_blank: true
+            requires :val, type: Integer, allow_blank: true
           end
-          get '/allow_fixnum_blank'
+          get '/allow_integer_blank'
 
           params do
             requires :val, type: Symbol, allow_blank: true
@@ -247,7 +252,7 @@ describe Grape::Validations::AllowBlankValidator do
 
   context 'invalid input' do
     it 'refuses empty string' do
-      get '/', name: ''
+      get '/disallow_blank', name: ''
       expect(last_response.status).to eq(400)
 
       get '/disallow_datetime_blank', val: ''
@@ -255,18 +260,23 @@ describe Grape::Validations::AllowBlankValidator do
     end
 
     it 'refuses only whitespaces' do
-      get '/', name: '   '
+      get '/disallow_blank', name: '   '
       expect(last_response.status).to eq(400)
 
-      get '/', name: "  \n "
+      get '/disallow_blank', name: "  \n "
       expect(last_response.status).to eq(400)
 
-      get '/', name: "\n"
+      get '/disallow_blank', name: "\n"
       expect(last_response.status).to eq(400)
     end
 
     it 'refuses nil' do
-      get '/', name: nil
+      get '/disallow_blank', name: nil
+      expect(last_response.status).to eq(400)
+    end
+
+    it 'refuses missing' do
+      get '/disallow_blank'
       expect(last_response.status).to eq(400)
     end
   end
@@ -335,8 +345,8 @@ describe Grape::Validations::AllowBlankValidator do
           expect(last_response.status).to eq(200)
         end
 
-        it 'accepts empty when fixnum allow_blank' do
-          get '/custom_message/allow_fixnum_blank', val: ''
+        it 'accepts empty when integer allow_blank' do
+          get '/custom_message/allow_integer_blank', val: ''
           expect(last_response.status).to eq(200)
         end
       end
@@ -432,8 +442,13 @@ describe Grape::Validations::AllowBlankValidator do
   end
 
   context 'valid input' do
+    it 'allows missing optional strings' do
+      get 'opt_disallow_string_blank'
+      expect(last_response.status).to eq(200)
+    end
+
     it 'accepts valid input' do
-      get '/', name: 'bob'
+      get '/disallow_blank', name: 'bob'
       expect(last_response.status).to eq(200)
     end
 
@@ -468,8 +483,8 @@ describe Grape::Validations::AllowBlankValidator do
         expect(last_response.status).to eq(200)
       end
 
-      it 'accepts empty when fixnum allow_blank' do
-        get '/allow_fixnum_blank', val: ''
+      it 'accepts empty when integer allow_blank' do
+        get '/allow_integer_blank', val: ''
         expect(last_response.status).to eq(200)
       end
     end
@@ -488,6 +503,11 @@ describe Grape::Validations::AllowBlankValidator do
       get '/disallow_boolean_blank', val: false
       expect(last_response.status).to eq(200)
     end
+
+    it 'accepts value when time allow_blank' do
+      get '/disallow_datetime_blank', val: Time.now
+      expect(last_response.status).to eq(200)
+    end
   end
 
   context 'in an optional group' do
