@@ -1,4 +1,3 @@
-# encoding: utf-8
 require 'spec_helper'
 
 describe Grape::Validations::CoerceValidator do
@@ -136,11 +135,11 @@ describe Grape::Validations::CoerceValidator do
         'array int works'
       end
 
-      get 'array', ids: %w(1 2 az)
+      get 'array', ids: %w[1 2 az]
       expect(last_response.status).to eq(400)
       expect(last_response.body).to eq('ids is invalid')
 
-      get 'array', ids: %w(1 2 890)
+      get 'array', ids: %w[1 2 890]
       expect(last_response.status).to eq(200)
       expect(last_response.body).to eq('array int works')
     end
@@ -175,7 +174,7 @@ describe Grape::Validations::CoerceValidator do
 
         get '/int', int: '45'
         expect(last_response.status).to eq(200)
-        expect(last_response.body).to eq('Fixnum')
+        expect(last_response.body).to eq(integer_class_name)
       end
 
       context 'Array' do
@@ -187,9 +186,9 @@ describe Grape::Validations::CoerceValidator do
             params[:arry][0].class
           end
 
-          get '/array', arry: %w(1 2 3)
+          get '/array', arry: %w[1 2 3]
           expect(last_response.status).to eq(200)
-          expect(last_response.body).to eq('Fixnum')
+          expect(last_response.body).to eq(integer_class_name)
         end
 
         it 'Array of Bools' do
@@ -225,6 +224,55 @@ describe Grape::Validations::CoerceValidator do
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('1')
         end
+
+        it 'Array of type implementing parse' do
+          subject.params do
+            requires :uri, type: Array[URI]
+          end
+          subject.get '/uri_array' do
+            params[:uri][0].class
+          end
+          get 'uri_array', uri: ['http://www.google.com']
+          expect(last_response.status).to eq(200)
+          expect(last_response.body).to eq('URI::HTTP')
+        end
+
+        it 'Set of type implementing parse' do
+          subject.params do
+            requires :uri, type: Set[URI]
+          end
+          subject.get '/uri_array' do
+            "#{params[:uri].class},#{params[:uri].first.class},#{params[:uri].size}"
+          end
+          get 'uri_array', uri: Array.new(2) { 'http://www.example.com' }
+          expect(last_response.status).to eq(200)
+          expect(last_response.body).to eq('Set,URI::HTTP,1')
+        end
+
+        it 'Array of class implementing parse and parsed?' do
+          class SecureURIOnly
+            def self.parse(value)
+              URI.parse(value)
+            end
+
+            def self.parsed?(value)
+              value.is_a? URI::HTTPS
+            end
+          end
+
+          subject.params do
+            requires :uri, type: Array[SecureURIOnly]
+          end
+          subject.get '/secure_uris' do
+            params[:uri].first.class
+          end
+          get 'secure_uris', uri: ['https://www.example.com']
+          expect(last_response.status).to eq(200)
+          expect(last_response.body).to eq('URI::HTTPS')
+          get 'secure_uris', uri: ['https://www.example.com', 'http://www.example.com']
+          expect(last_response.status).to eq(400)
+          expect(last_response.body).to eq('uri is invalid')
+        end
       end
 
       context 'Set' do
@@ -238,7 +286,7 @@ describe Grape::Validations::CoerceValidator do
 
           get '/set', set: Set.new([1, 2, 3, 4]).to_a
           expect(last_response.status).to eq(200)
-          expect(last_response.body).to eq('Fixnum')
+          expect(last_response.body).to eq(integer_class_name)
         end
 
         it 'Set of Bools' do
@@ -280,12 +328,49 @@ describe Grape::Validations::CoerceValidator do
         expect(last_response.body).to eq('TrueClass')
       end
 
+      it 'Boolean' do
+        subject.params do
+          optional :boolean, type: Boolean, default: true
+        end
+        subject.get '/boolean' do
+          params[:boolean].class
+        end
+
+        get '/boolean'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('TrueClass')
+
+        get '/boolean', boolean: true
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('TrueClass')
+
+        get '/boolean', boolean: false
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('FalseClass')
+
+        get '/boolean', boolean: 'true'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('TrueClass')
+
+        get '/boolean', boolean: 'false'
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('FalseClass')
+
+        get '/boolean', boolean: 123
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('boolean is invalid')
+
+        get '/boolean', boolean: '123'
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('boolean is invalid')
+      end
+
       it 'Rack::Multipart::UploadedFile' do
         subject.params do
           requires :file, type: Rack::Multipart::UploadedFile
         end
         subject.post '/upload' do
-          params[:file].filename
+          params[:file][:filename]
         end
 
         post '/upload', file: Rack::Test::UploadedFile.new(__FILE__)
@@ -302,7 +387,7 @@ describe Grape::Validations::CoerceValidator do
           requires :file, coerce: File
         end
         subject.post '/upload' do
-          params[:file].filename
+          params[:file][:filename]
         end
 
         post '/upload', file: Rack::Test::UploadedFile.new(__FILE__)
@@ -326,7 +411,7 @@ describe Grape::Validations::CoerceValidator do
 
         get '/int', integers: { int: '45' }
         expect(last_response.status).to eq(200)
-        expect(last_response.body).to eq('Fixnum')
+        expect(last_response.body).to eq(integer_class_name)
       end
     end
 
@@ -358,11 +443,11 @@ describe Grape::Validations::CoerceValidator do
 
         get '/ints', values: '1 2 3 4'
         expect(last_response.status).to eq(200)
-        expect(JSON.parse(last_response.body)).to eq(%w(1 2 3 4))
+        expect(JSON.parse(last_response.body)).to eq(%w[1 2 3 4])
 
         get '/ints', values: 'a b c d'
         expect(last_response.status).to eq(200)
-        expect(JSON.parse(last_response.body)).to eq(%w(0 0 0 0))
+        expect(JSON.parse(last_response.body)).to eq(%w[0 0 0 0])
       end
 
       it 'parses parameters with Array[Integer] type' do
@@ -382,6 +467,23 @@ describe Grape::Validations::CoerceValidator do
         expect(JSON.parse(last_response.body)).to eq([0, 0, 0, 0])
       end
 
+      it 'parses parameters even if type is valid' do
+        subject.params do
+          requires :values, type: Array, coerce_with: ->(array) { array.map { |val| val.to_i + 1 } }
+        end
+        subject.get '/ints' do
+          params[:values]
+        end
+
+        get '/ints', values: [1, 2, 3, 4]
+        expect(last_response.status).to eq(200)
+        expect(JSON.parse(last_response.body)).to eq([2, 3, 4, 5])
+
+        get '/ints', values: %w[a b c d]
+        expect(last_response.status).to eq(200)
+        expect(JSON.parse(last_response.body)).to eq([1, 1, 1, 1])
+      end
+
       it 'uses parse where available' do
         subject.params do
           requires :ints, type: Array, coerce_with: JSON do
@@ -400,7 +502,7 @@ describe Grape::Validations::CoerceValidator do
 
         get '/ints', ints: '{"i":1,"j":"2"}'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('ints[0][i] is missing, ints[0][i] is invalid, ints[0][j] is missing')
+        expect(last_response.body).to eq('ints is invalid')
 
         get '/ints', ints: '[{"i":"1","j":"2"}]'
         expect(last_response.status).to eq(200)
@@ -440,7 +542,7 @@ describe Grape::Validations::CoerceValidator do
     end
 
     context 'first-class JSON' do
-      it 'parses objects and arrays' do
+      it 'parses objects, hashes, and arrays' do
         subject.params do
           requires :splines, type: JSON do
             requires :x, type: Integer, values: [1, 2, 3]
@@ -453,8 +555,8 @@ describe Grape::Validations::CoerceValidator do
         subject.get '/' do
           if params[:splines].is_a? Hash
             params[:splines][:obj][:y]
-          else
-            'arrays work' if params[:splines].any? { |s| s.key? :obj }
+          elsif params[:splines].any? { |s| s.key? :obj }
+            'arrays work'
           end
         end
 
@@ -462,17 +564,33 @@ describe Grape::Validations::CoerceValidator do
         expect(last_response.status).to eq(200)
         expect(last_response.body).to eq('woof')
 
+        get '/', splines: { x: 1, ints: [1, 2, 3], obj: { y: 'woof' } }
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('woof')
+
         get '/', splines: '[{"x":2,"ints":[]},{"x":3,"ints":[4],"obj":{"y":"quack"}}]'
         expect(last_response.status).to eq(200)
         expect(last_response.body).to eq('arrays work')
 
+        get '/', splines: [{ x: 2, ints: [] }, { x: 3, ints: [4], obj: { y: 'quack' } }]
+        expect(last_response.status).to eq(200)
+        expect(last_response.body).to eq('arrays work')
+
         get '/', splines: '{"x":4,"ints":[2]}'
         expect(last_response.status).to eq(400)
         expect(last_response.body).to eq('splines[x] does not have a valid value')
 
+        get '/', splines: { x: 4, ints: [2] }
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('splines[x] does not have a valid value')
+
         get '/', splines: '[{"x":1,"ints":[]},{"x":4,"ints":[]}]'
         expect(last_response.status).to eq(400)
         expect(last_response.body).to eq('splines[x] does not have a valid value')
+
+        get '/', splines: [{ x: 1, ints: [] }, { x: 4, ints: [] }]
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('splines[x] does not have a valid value')
       end
 
       it 'works when declared optional' do
@@ -488,8 +606,8 @@ describe Grape::Validations::CoerceValidator do
         subject.get '/' do
           if params[:splines].is_a? Hash
             params[:splines][:obj][:y]
-          else
-            'arrays work' if params[:splines].any? { |s| s.key? :obj }
+          elsif params[:splines].any? { |s| s.key? :obj }
+            'arrays work'
           end
         end
 
@@ -525,11 +643,11 @@ describe Grape::Validations::CoerceValidator do
 
         get '/', splines: '{"x":"1","y":"woof"}'
         expect(last_response.status).to eq(200)
-        expect(last_response.body).to eq('Fixnum.String')
+        expect(last_response.body).to eq("#{integer_class_name}.String")
 
         get '/', splines: '[{"x":1,"y":2},{"x":1,"y":"quack"}]'
         expect(last_response.status).to eq(200)
-        expect(last_response.body).to eq('Fixnum.Fixnum')
+        expect(last_response.body).to eq("#{integer_class_name}.#{integer_class_name}")
 
         get '/', splines: '{"x":"4","y":"woof"}'
         expect(last_response.status).to eq(400)
@@ -576,7 +694,7 @@ describe Grape::Validations::CoerceValidator do
 
         get '/', a: '5'
         expect(last_response.status).to eq(200)
-        expect(last_response.body).to eq('Fixnum')
+        expect(last_response.body).to eq(integer_class_name)
 
         get '/', a: 'anything else'
         expect(last_response.status).to eq(200)
@@ -623,7 +741,7 @@ describe Grape::Validations::CoerceValidator do
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('"one way"')
 
-          get '/', a: %w(the other)
+          get '/', a: %w[the other]
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('["the", "other"]')
 
@@ -641,7 +759,7 @@ describe Grape::Validations::CoerceValidator do
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('[1, 2, 3]')
 
-          get '/', b: %w(1 2 3)
+          get '/', b: %w[1 2 3]
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('[1, 2, 3]')
 
@@ -663,12 +781,84 @@ describe Grape::Validations::CoerceValidator do
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('"one"')
 
-          get '/', d: %w(1 two)
+          get '/', d: %w[1 two]
           expect(last_response.status).to eq(200)
           expect(last_response.body).to eq('#<Set: {1, "two"}>')
         end
       end
 
+      context 'when params is Hashie::Mash' do
+        context 'for primitive collections' do
+          before do
+            subject.params do
+              build_with Grape::Extensions::Hashie::Mash::ParamBuilder
+              optional :a, types: [String, Array[String]]
+              optional :b, types: [Array[Integer], Array[String]]
+              optional :c, type: Array[Integer, String]
+              optional :d, types: [Integer, String, Set[Integer, String]]
+            end
+            subject.get '/' do
+              (
+                params.a ||
+                params.b ||
+                params.c ||
+                params.d
+              ).inspect
+            end
+          end
+
+          it 'allows singular form declaration' do
+            get '/', a: 'one way'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('"one way"')
+
+            get '/', a: %w[the other]
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array ["the", "other"]>')
+
+            get '/', a: { a: 1, b: 2 }
+            expect(last_response.status).to eq(400)
+            expect(last_response.body).to eq('a is invalid')
+
+            get '/', a: [1, 2, 3]
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array ["1", "2", "3"]>')
+          end
+
+          it 'allows multiple collection types' do
+            get '/', b: [1, 2, 3]
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array [1, 2, 3]>')
+
+            get '/', b: %w[1 2 3]
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array [1, 2, 3]>')
+
+            get '/', b: [1, true, 'three']
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array ["1", "true", "three"]>')
+          end
+
+          it 'allows collections with multiple types' do
+            get '/', c: [1, '2', true, 'three']
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Hashie::Array [1, 2, "true", "three"]>')
+
+            get '/', d: '1'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('1')
+
+            get '/', d: 'one'
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('"one"')
+
+            get '/', d: %w[1 two]
+            expect(last_response.status).to eq(200)
+            expect(last_response.body).to eq('#<Set: {1, "two"}>')
+          end
+        end
+      end
+
       context 'custom coercion rules' do
         before do
           subject.params do
