@@ -63,7 +63,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_optional_using
-        expect(subject.route_setting(:declared_params)).to eq([:field_a, :field_b])
+        expect(subject.route_setting(:declared_params)).to eq(%i[field_a field_b])
       end
 
       it 'works when field_a and field_b are not present' do
@@ -139,7 +139,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_requires_all
-        expect(subject.route_setting(:declared_params)).to eq([:required_field, :optional_field])
+        expect(subject.route_setting(:declared_params)).to eq(%i[required_field optional_field])
       end
 
       it 'errors when required_field is not present' do
@@ -174,7 +174,7 @@ describe Grape::Validations do
 
       it 'adds entity documentation to declared params' do
         define_requires_none
-        expect(subject.route_setting(:declared_params)).to eq([:required_field, :optional_field])
+        expect(subject.route_setting(:declared_params)).to eq(%i[required_field optional_field])
       end
 
       it 'errors when required_field is not present' do
@@ -204,7 +204,7 @@ describe Grape::Validations do
 
         it 'adds only the entity documentation to declared params, nothing more' do
           define_requires_all
-          expect(subject.route_setting(:declared_params)).to eq([:required_field, :optional_field])
+          expect(subject.route_setting(:declared_params)).to eq(%i[required_field optional_field])
         end
       end
 
@@ -245,7 +245,7 @@ describe Grape::Validations do
       it 'errors when param is not an Array' do
         get '/required', items: 'hello'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items is invalid, items[key] is missing')
+        expect(last_response.body).to eq('items is invalid')
 
         get '/required', items: { key: 'foo' }
         expect(last_response.status).to eq(400)
@@ -274,6 +274,28 @@ describe Grape::Validations do
       end
     end
 
+    # Ensure there is no leakage between declared Array types and
+    # subsequent Hash types
+    context 'required with an Array and a Hash block' do
+      before do
+        subject.params do
+          requires :cats, type: Array[String], default: []
+          requires :items, type: Hash do
+            requires :key
+          end
+        end
+        subject.get '/required' do
+          'required works'
+        end
+      end
+
+      it 'does not output index [0] for Hash types' do
+        get '/required', cats: ['Garfield'], items: { foo: 'bar' }
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('items[key] is missing')
+      end
+    end
+
     context 'required with a Hash block' do
       before do
         subject.params do
@@ -292,6 +314,12 @@ describe Grape::Validations do
         expect(last_response.body).to eq('items is missing, items[key] is missing')
       end
 
+      it 'errors when nested param not present' do
+        get '/required', items: { foo: 'bar' }
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('items[key] is missing')
+      end
+
       it 'errors when param is not a Hash' do
         get '/required', items: 'hello'
         expect(last_response.status).to eq(400)
@@ -322,7 +350,7 @@ describe Grape::Validations do
       before do
         subject.params do
           requires :items, type: Hash do
-            requires :key, type: String, values: %w(a b)
+            requires :key, type: String, values: %w[a b]
           end
         end
         subject.get '/required' do
@@ -337,7 +365,7 @@ describe Grape::Validations do
 
         get '/required', items: [{ key: 'hash in array' }]
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items is invalid, items[0][key] does not have a valid value')
+        expect(last_response.body).to eq('items is invalid, items[key] does not have a valid value')
       end
 
       it 'works when all params match' do
@@ -396,7 +424,7 @@ describe Grape::Validations do
         end
       end
 
-      it 'errors when group param is invalid'do
+      it 'errors when group param is invalid' do
         post '/group_with_nested', items: invalid_items
         expect(last_response.status).to eq(400)
       end
@@ -408,7 +436,7 @@ describe Grape::Validations do
           class DateRangeValidator < Grape::Validations::Base
             def validate_param!(attr_name, params)
               return if params[attr_name][:from] <= params[attr_name][:to]
-              fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: "'from' must be lower or equal to 'to'"
+              raise Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: "'from' must be lower or equal to 'to'"
             end
           end
         end
@@ -466,7 +494,7 @@ describe Grape::Validations do
           group :children, type: Array do
             requires :name
             group :parents, type: Array do
-              requires :name
+              requires :name, allow_blank: false
             end
           end
         end
@@ -486,13 +514,33 @@ describe Grape::Validations do
 
       it 'errors when a parameter is not present' do
         get '/within_array', children: [
-          { name: 'Jim', parents: [{}] },
-          { name: 'Job', parents: [{ name: 'Joy' }] }
+          { name: 'Jim', parents: [{ name: 'Joy' }] },
+          { name: 'Job', parents: [{}] }
         ]
         # NOTE: with body parameters in json or XML or similar this
         # should actually fail with: children[parents][name] is missing.
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children[0][parents] is missing')
+        expect(last_response.body).to eq('children[1][parents] is missing')
+      end
+
+      it 'errors when a parameter is not present in array within array' do
+        get '/within_array', children: [
+          { name: 'Jim', parents: [{ name: 'Joy' }] },
+          { name: 'Job', parents: [{ name: 'Bill' }, { name: '' }] }
+        ]
+
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('children[1][parents][1][name] is empty')
+      end
+
+      it 'handle errors for all array elements' do
+        get '/within_array', children: [
+          { name: 'Jim', parents: [] },
+          { name: 'Job', parents: [] }
+        ]
+
+        expect(last_response.status).to eq(400)
+        expect(last_response.body).to eq('children[0][parents] is missing, children[1][parents] is missing')
       end
 
       it 'safely handles empty arrays and blank parameters' do
@@ -507,14 +555,13 @@ describe Grape::Validations do
       end
 
       it 'errors when param is not an Array' do
-        # NOTE: would be nicer if these just returned 'children is invalid'
         get '/within_array', children: 'hello'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children is invalid, children[name] is missing, children[parents] is missing, children[parents] is invalid, children[parents][name] is missing')
+        expect(last_response.body).to eq('children is invalid')
 
         get '/within_array', children: { name: 'foo' }
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('children is invalid, children[parents] is missing')
+        expect(last_response.body).to eq('children is invalid')
 
         get '/within_array', children: [name: 'Jay', parents: { name: 'Fred' }]
         expect(last_response.status).to eq(400)
@@ -565,7 +612,7 @@ describe Grape::Validations do
       it 'requires defaults to Array type' do
         get '/req', planets: 'Jupiter, Saturn'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('planets is invalid, planets[name] is missing')
+        expect(last_response.body).to eq('planets is invalid')
 
         get '/req', planets: { name: 'Jupiter' }
         expect(last_response.status).to eq(400)
@@ -581,7 +628,7 @@ describe Grape::Validations do
       it 'optional defaults to Array type' do
         get '/opt', name: 'Jupiter', moons: 'Europa, Ganymede'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('moons is invalid, moons[name] is missing')
+        expect(last_response.body).to eq('moons is invalid')
 
         get '/opt', name: 'Jupiter', moons: { name: 'Ganymede' }
         expect(last_response.status).to eq(400)
@@ -600,7 +647,7 @@ describe Grape::Validations do
       it 'group defaults to Array type' do
         get '/grp', stars: 'Sun'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('stars is invalid, stars[name] is missing')
+        expect(last_response.body).to eq('stars is invalid')
 
         get '/grp', stars: { name: 'Sun' }
         expect(last_response.status).to eq(400)
@@ -689,7 +736,7 @@ describe Grape::Validations do
       it "errors when param is present but isn't an Array" do
         get '/optional_group', items: 'hello'
         expect(last_response.status).to eq(400)
-        expect(last_response.body).to eq('items is invalid, items[key] is missing')
+        expect(last_response.body).to eq('items is invalid')
 
         get '/optional_group', items: { key: 'foo' }
         expect(last_response.status).to eq(400)
@@ -794,7 +841,7 @@ describe Grape::Validations do
         class Customvalidator < Grape::Validations::Base
           def validate_param!(attr_name, params)
             return if params[attr_name] == 'im custom'
-            fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: 'is not custom!'
+            raise Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: 'is not custom!'
           end
         end
       end
@@ -942,7 +989,7 @@ describe Grape::Validations do
           class CustomvalidatorWithOptions < Grape::Validations::Base
             def validate_param!(attr_name, params)
               return if params[attr_name] == @option[:text]
-              fail Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message
+              raise Grape::Exceptions::Validation, params: [@scope.full_name(attr_name)], message: message
             end
           end
         end
@@ -1011,14 +1058,14 @@ describe Grape::Validations do
           subject.params do
             use :pagination
           end
-          expect(subject.route_setting(:declared_params)).to eq [:page, :per_page]
+          expect(subject.route_setting(:declared_params)).to eq %i[page per_page]
         end
 
         it 'by #use with multiple params' do
           subject.params do
             use :pagination, :period
           end
-          expect(subject.route_setting(:declared_params)).to eq [:page, :per_page, :start_date, :end_date]
+          expect(subject.route_setting(:declared_params)).to eq %i[page per_page start_date end_date]
         end
       end
 
@@ -1026,13 +1073,13 @@ describe Grape::Validations do
         before do
           subject.helpers do
             params :order do |options|
-              optional :order, type: Symbol, values: [:asc, :desc], default: options[:default_order]
+              optional :order, type: Symbol, values: %i[asc desc], default: options[:default_order]
               optional :order_by, type: Symbol, values: options[:order_by], default: options[:default_order_by]
             end
           end
           subject.format :json
           subject.params do
-            use :order, default_order: :asc, order_by: [:name, :created_at], default_order_by: :created_at
+            use :order, default_order: :asc, order_by: %i[name created_at], default_order_by: :created_at
           end
           subject.get '/order' do
             {
