@@ -6,14 +6,7 @@ require 'grape'
 
 require 'rubygems'
 require 'bundler'
-Bundler.setup :default, :test
-
-require 'json'
-require 'rack/test'
-require 'base64'
-require 'cookiejar'
-require 'mime/types'
-require 'cgi'
+Bundler.require :default, :test
 
 Dir["#{File.dirname(__FILE__)}/support/*.rb"].each do |file|
   require file
@@ -23,7 +16,11 @@ I18n.enforce_available_locales = false
 
 RSpec.configure do |config|
   config.include Rack::Test::Methods
+  config.include Spec::Support::Helpers
   config.raise_errors_for_deprecations!
 
   config.before(:each) { Grape::Util::InheritableSetting.reset_global! }
 end
+
+require 'coveralls'
+Coveralls.wear!
